declare global {
    var ContactForm: {
        endpoint: string;
        hashKey: string;
        hashValue: string;
        intro: string;
    };
}

export interface Retailer {
    id: number;
    name: string;
    street_address_1: string;
    street_address_2: string;
    suburb: string;
    state: string;
    postcode: string;
    country: string;
    phone: string;
    email: string;
    website: string;
    types: RetailerType[];
    services: string[];
    products: string[];
    latitude: number;
    longitude: number;
    status: string;
    creation_date: string;
}

export type RetailerType =
    'Mobile Consultant'
    | 'Showroom'
    | 'Repair'

export interface SearchResult {
    retailer: Retailer;
    counter: string;
    selected: boolean;
}

export interface FreeformFields {
    name: string;
    email: string;
    location: string;
    type_of_contact: 'user' | 'retailer' | 'installer';
    type_of_enquiry: 'sales' | 'technical' | 'other';
    retailers: number[];
    retailer_recipients: string[];
    message: string;
}

export type FreeformErrors = Partial<Record<keyof FreeformFields, string[]>>;

export type FreeformHoneypot = {
    name: string;
    hash: string;
}

export type FreeformSuccessResponse = {
    success: true;
    finished: true;
    returnUrl: string;
    submissionId: number | null;
    honeypot: FreeformHoneypot
}

export type FreeformErrorResponse = {
    success: false;
    finished: false;
    formErrors: string[];
    errors: FreeformErrors;
    honeypot: FreeformHoneypot
}

export type FreeformResponse = FreeformSuccessResponse | FreeformErrorResponse;
