import {defineStore} from 'pinia';
import {Image, Tab} from './types';

interface State {
    tabs: Tab[];
    images: Image[];
    currentTab: string;
    currentIndex?: number;
}

export const useStore = defineStore('global', {
    state: () => ({
        tabs: window.Gallery.tabs,
        images: window.Gallery.images,
        currentTab: window.Gallery.tabs[0].id,
        currentIndex: undefined,
    } as State),
    getters: {
        galleryImages(state): Image[] {
            return state.images
                .filter(i => i.tab === state.currentTab);
        },
        currentImage(state): (Image & { index: number, total: number }) | undefined {
            if (state.currentIndex !== undefined) {
                const image = this.galleryImages[state.currentIndex];
                return {
                    ...image,
                    index: state.currentIndex + 1,
                    total: this.galleryImages.length,
                };
            }
        },
    },
    actions: {
        setCurrentTab(id: string) {
            this.currentTab = id;
        },
        setCurrentImage(index?: number) {
            this.currentIndex = index;
        },
    },
});
