import {defineStore} from 'pinia';
import type {FreeformErrors} from './types';

interface State {
    view: 'form' | 'success';
    formCompany: string | null;
    formStreet1: string | null;
    formStreet2: string | null;
    formSuburb: string | null;
    formState: string | null;
    formPostcode: string | null;
    formPhone: string | null;
    formEmail: string | null;
    formWebsite: string | null;
    formType: string[];
    formService: string[];
    formProducts: string[];
    formMessage: string | null;
    errorCompany: string | null;
    errorStreet1: string | null;
    errorStreet2: string | null;
    errorSuburb: string | null;
    errorState: string | null;
    errorPostcode: string | null;
    errorPhone: string | null;
    errorEmail: string | null;
    errorWebsite: string | null;
    errorType: string | null;
    errorService: string | null;
    errorProducts: string | null;
    errorMessage: string | null;
}

export const useStore = defineStore('global', {
    state: () => ({
        view: 'form',
        formCompany: null,
        formStreet1: null,
        formStreet2: null,
        formSuburb: null,
        formState: null,
        formPostcode: null,
        formPhone: null,
        formEmail: null,
        formWebsite: null,
        formType: [],
        formService: [],
        formProducts: [],
        formMessage: null,
        errorCompany: null,
        errorStreet1: null,
        errorStreet2: null,
        errorSuburb: null,
        errorState: null,
        errorPostcode: null,
        errorPhone: null,
        errorEmail: null,
        errorWebsite: null,
        errorType: null,
        errorService: null,
        errorProducts: null,
        errorMessage: null,
    } as State),
    getters: {
        hasErrors(state): boolean {
            return state.errorCompany !== null
                   || state.errorStreet1 !== null
                   || state.errorStreet2 !== null
                   || state.errorSuburb !== null
                   || state.errorState !== null
                   || state.errorPostcode !== null
                   || state.errorPhone !== null
                   || state.errorEmail !== null
                   || state.errorWebsite !== null
                   || state.errorType !== null
                   || state.errorService !== null
                   || state.errorProducts !== null
                   || state.errorMessage !== null;
        },
    },
    actions: {
        resetErrors() {
            this.errorCompany = null;
            this.errorStreet1 = null;
            this.errorStreet2 = null;
            this.errorSuburb = null;
            this.errorState = null;
            this.errorPostcode = null;
            this.errorPhone = null;
            this.errorEmail = null;
            this.errorWebsite = null;
            this.errorType = null;
            this.errorService = null;
            this.errorProducts = null;
            this.errorMessage = null;
        },
        validate() {
            this.resetErrors();

            if (this.formCompany == null || this.formCompany.trim() === '') {
                this.errorCompany = 'This field is required.';
            }
            if (this.formStreet1 == null || this.formStreet1.trim() === '') {
                this.errorStreet1 = 'This field is required.';
            }
            if (this.formSuburb == null || this.formSuburb.trim() === '') {
                this.errorSuburb = 'This field is required.';
            }
            if (this.formState == null || this.formState.trim() === '') {
                this.errorState = 'This field is required.';
            }
            if (this.formPostcode == null || this.formPostcode.trim() === '') {
                this.errorPostcode = 'This field is required.';
            }
            if (this.formEmail == null || this.formEmail.trim() === '') {
                this.errorEmail = 'This field is required.';
            }

            return this.hasErrors;
        },
        setResponseErrors(errors: FreeformErrors) {
            this.resetErrors();

            if (errors['company']?.length) {
                this.errorCompany = errors['company'][0];
            }
            if (errors['street_1']?.length) {
                this.errorStreet1 = errors['street_1'][0];
            }
            if (errors['street_2']?.length) {
                this.errorStreet2 = errors['street_2'][0];
            }
            if (errors['suburb']?.length) {
                this.errorSuburb = errors['suburb'][0];
            }
            if (errors['state']?.length) {
                this.errorState = errors['state'][0];
            }
            if (errors['postcode']?.length) {
                this.errorPostcode = errors['postcode'][0];
            }
            if (errors['phone']?.length) {
                this.errorPhone = errors['phone'][0];
            }
            if (errors['email']?.length) {
                this.errorEmail = errors['email'][0];
            }
            if (errors['website']?.length) {
                this.errorWebsite = errors['website'][0];
            }
            if (errors['type']?.length) {
                this.errorType = errors['type'][0];
            }
            if (errors['service']?.length) {
                this.errorService = errors['service'][0];
            }
            if (errors['products']?.length) {
                this.errorProducts = errors['products'][0];
            }
            if (errors['message']?.length) {
                this.errorMessage = errors['message'][0];
            }
        },
    },
});
