const R = 6371.0710; // Radius of earth in km

const toRadian = (deg: number) => deg * (Math.PI / 180);

type LatLngPair = { lat: number, lng: number };

// Calculate distance using the Haversine formula
export default function calculateGeoDistance(a: LatLngPair, b: LatLngPair) {
    const {asin, sqrt, sin, cos, round} = Math;

    const rlata = toRadian(a.lat);
    const rlatb = toRadian(b.lat);
    const dlat = toRadian(b.lat - a.lat);
    const dlng = toRadian(b.lng - a.lng);

    return round(2 * R * asin(sqrt(sin(dlat / 2) * sin(dlat / 2) + cos(rlata) * cos(rlatb) * sin(dlng / 2) * sin(dlng / 2))));
}
