declare global {
    var Locator: {
        endpoint: string;
        hashKey: string;
        hashValue: string;
        searchTitle: string;
        searchIntro: string;
        calloutUrl: string;
        calloutImage: string;
        calloutContent: string;
    };
}

export interface Retailer {
    id: number;
    name: string;
    street_address_1: string;
    street_address_2: string;
    suburb: string;
    state: string;
    postcode: string;
    country: string;
    phone: string;
    email: string;
    website: string;
    types: RetailerType[];
    services: RetailerService[];
    latitude: number;
    longitude: number;
}

export type RetailerType = 'Mobile Consultant' | 'Showroom' | 'Repair'

export type RetailerService = 'New Sales' | 'Repairs'

export interface SearchResult {
    retailer: Retailer;
    distance: number;
    counter: string;
    selected: boolean;
}

export interface FreeformFields {
    name: string;
    phone: string;
    email: string;
    retailers: number[];
    retailer_recipients: string[];
    message: string;
}

export type FreeformErrors = Partial<Record<keyof FreeformFields, string[]>>;

export type FreeformHoneypot = {
    name: string;
    hash: string;
}

export type FreeformSuccessResponse = {
    success: true;
    finished: true;
    returnUrl: string;
    submissionId: number | null;
    honeypot: FreeformHoneypot
}

export type FreeformErrorResponse = {
    success: false;
    finished: false;
    formErrors: string[];
    errors: FreeformErrors;
    honeypot: FreeformHoneypot
}

export type FreeformResponse = FreeformSuccessResponse | FreeformErrorResponse;
