import getProjectConfig from '../config.task';

export default function getTaskConfig(tasks) {
    const finalConfig = {};
    const projectConfig = getProjectConfig();

    for (const name of Object.keys(tasks)) {
        const task = tasks[name].default;
        const configName = task.configName || name;
        const taskConfig = task.defaultConfig();
        const userConfig = (projectConfig[configName] || {});

        finalConfig[configName] = deepMerge(taskConfig, userConfig);
    }

    return finalConfig;
}

function isObject(item) {
    return item
        && typeof item === 'object'
        && !Array.isArray(item);
}

function deepMerge(target, source) {
    let output = Object.assign({}, target);
    if (isObject(target) && isObject(source)) {
        for (const [key, value] of Object.entries(source)) {
            if (isObject(value)) {
                if (!(key in target)) {
                    Object.assign(output, {[key]: value});
                } else {
                    output[key] = deepMerge(target[key], value);
                }
            } else {
                Object.assign(output, {[key]: value});
            }
        }
    }
    return output;
}
