import deasync from 'deasync';
import fs from 'fs';
import imagemin from 'imagemin';
import imageminGifsicle from 'imagemin-gifsicle';
import imageminMozjpeg from 'imagemin-mozjpeg';
import imageminOptipng from 'imagemin-optipng';
import mimeTypes from 'mime-types';
import {types} from 'sass';
import projectPath from '../lib/projectPath';

export default {
    declaration: 'data-url($path)',
    implementation: dataUrl,
};

const imageminConfig = () => ({
    plugins: [
        imageminGifsicle(TASK_CONFIG.images.gifsicle),
        imageminMozjpeg(TASK_CONFIG.images.mozjpeg),
        imageminOptipng(TASK_CONFIG.images.optipng),
    ],
});

function dataUrl(imagePath) {
    const filePath = projectPath(PATH_CONFIG.src, PATH_CONFIG.styles.src, imagePath.getValue());
    const mimeType = mimeTypes.lookup(filePath);
    const buffer = deasync(optimizeImage)(fs.readFileSync(filePath));

    return new types.String(`url("data:${mimeType};base64,${buffer.toString('base64')}")`);
}

function optimizeImage(buffer, cb) {
    imagemin.buffer(buffer, imageminConfig())
        .then((result) => cb(null, result));
}
