import sizeOf from 'image-size';
import {types} from 'sass';
import projectPath from '../lib/projectPath';

export default [
    {
        declaration: 'img-size($basename)',
        implementation: imageSize,
    },
    {
        declaration: 'img-width($basename)',
        implementation: imageWidth,
    },
    {
        declaration: 'img-height($basename)',
        implementation: imageHeight,
    },
];

const sizeCache = {};
const imagesPath = projectPath(PATH_CONFIG.src, PATH_CONFIG.images.src);

function imageSize(basename) {
    const size = new types.List(2);
    size.setValue(0, imageWidth(basename));
    size.setValue(1, imageHeight(basename));
    return size;
}

function imageWidth(basename) {
    const {width} = rawImageSize(basename);
    return new types.Number(width, 'px');
}

function imageHeight(basename) {
    const {height} = rawImageSize(basename);
    return new types.Number(height, 'px');
}

function rawImageSize(basename) {
    if (!sizeCache.hasOwnProperty(basename)) {
        const name = basename.getValue();
        const path = `${imagesPath}/${name}`;
        const {width, height} = sizeOf(path);
        sizeCache[basename] = {width, height};
    }
    return sizeCache[basename];
}
