import gulp from 'gulp';
import changed from 'gulp-changed';
import imagemin from 'gulp-imagemin';
import projectPath from '../lib/projectPath';
import {sync} from './browser-sync';

export default function images() {
    const src = projectPath(PATH_CONFIG.src, PATH_CONFIG.images.src, '**', '*.{' + TASK_CONFIG.images.extensions + '}');
    const dest = projectPath(PATH_CONFIG.dest, PATH_CONFIG.images.dest);

    return gulp.src(src)
        .pipe(changed(dest))
        .pipe(imagemin([
            imagemin.gifsicle(TASK_CONFIG.images.gifsicle),
            imagemin.mozjpeg(TASK_CONFIG.images.mozjpeg),
            imagemin.optipng(TASK_CONFIG.images.optipng),
            imagemin.svgo(TASK_CONFIG.images.svgo),
        ]))
        .pipe(gulp.dest(dest))
        .pipe(sync.stream());
}

images.displayName = 'images';
images.description = 'Compress and optimise images.';
images.defaultConfig = () => ({
    extensions: ['gif', 'jpg', 'jpeg', 'png', 'svg'],
    gifsicle: {optimizationLevel: 3},
    mozjpeg: {},
    optipng: {optimizationLevel: 7},
    svgo: {
        plugins: [
            {name: 'cleanupIDs'},
            {name: 'removeViewBox'},
            {name: 'removeUselessDefs'},
        ],
    },
});
images.register = () => {
    gulp.task(images);
};
