import gulp from 'gulp';
import rename from 'gulp-rename';
import rev from 'gulp-rev';
import override from 'gulp-rev-css-url';
import projectPath from '../lib/projectPath';

export default function revTask() {
    const dest = projectPath(PATH_CONFIG.dest);

    return gulp.src(TASK_CONFIG.rev.paths, {base: dest})
        .pipe(rev())
        .pipe(override())
        .pipe(rename(TASK_CONFIG.rev.rename))
        .pipe(gulp.dest(dest))
        .pipe(rev.manifest(TASK_CONFIG.rev.manifest))
        .pipe(gulp.dest(dest));
}

revTask.displayName = 'rev';
revTask.description = 'Revision assets for production.';
revTask.defaultConfig = () => ({
    paths: [
        projectPath(PATH_CONFIG.dest, PATH_CONFIG.images.dest, '**'),
        projectPath(PATH_CONFIG.dest, PATH_CONFIG.styles.dest, '*.css'),
        projectPath(PATH_CONFIG.dest, PATH_CONFIG.scripts.dest, '*.js'),
    ],
    rename: (pathname) => {
        if (/\.(css|js)$/i.test(pathname.extname)) {
            pathname.extname = `.min${pathname.extname}`;
        }
    },
    manifest: {
        path: 'asset-manifest.json',
        transformer: {
            parse(contents) {
                return JSON.parse(contents);
            },
            stringify(contents, replacer, space) {
                const output = {};
                for (const key of Object.keys(contents)) {
                    output[`assets/${key}`] = `assets/${contents[key]}`;
                }
                return JSON.stringify(output, replacer, space);
            },
        },
    },
});
revTask.register = () => {
    gulp.task(revTask);
};
