import gulp, {parallel} from 'gulp';
import concat from 'gulp-concat';
import gulpIf from 'gulp-if';
import sourcemaps from 'gulp-sourcemaps';
import uglify from 'gulp-uglify';
import projectPath from '../lib/projectPath';
import {sync} from './browser-sync';

const subtasks = [];
export default function scripts(done) {
    return subtasks.length
        ? parallel(subtasks)(done)
        : done();
}

const scriptsSubtask = (src, target) => () => {
    const dest = projectPath(PATH_CONFIG.dest, PATH_CONFIG.scripts.dest);

    return gulp.src(src)
        .pipe(gulpIf(GULP_ENV.debug, sourcemaps.init()))
        .pipe(concat(target))
        .pipe(gulpIf(GULP_ENV.prod, uglify(TASK_CONFIG.scripts.uglify)))
        .pipe(gulpIf(GULP_ENV.debug, sourcemaps.write()))
        .pipe(gulp.dest(dest))
        .pipe(sync.stream());
};

scripts.displayName = 'scripts';
scripts.description = 'Concatenate and optimise scripts.';
scripts.defaultConfig = () => ({
    uglify: {},
});
scripts.register = () => {
    gulp.task(scripts);

    for (const key of Object.keys(TASK_CONFIG.scripts)) {
        if (key === 'uglify') {
            continue;
        }
        gulp.task(`scripts:${key}`, scriptsSubtask(TASK_CONFIG.scripts[key], key));
        subtasks.push(`scripts:${key}`);
    }
};
