import gulp from 'gulp';
import rename from 'gulp-rename';
import svgmin from 'gulp-svgmin';
import svgstore from 'gulp-svgstore';
import path from 'path';
import projectPath from '../lib/projectPath';
import {sync} from './browser-sync';

export default function sprites() {
    const src = projectPath(PATH_CONFIG.src, PATH_CONFIG.sprites.src, '*.{' + TASK_CONFIG.sprites.extensions + '}');
    const dest = TASK_CONFIG.sprites.dest || projectPath(PATH_CONFIG.dest, PATH_CONFIG.sprites.dest);

    return gulp.src(src, {base: path.dirname(src)})
        .pipe(svgmin((file) => {
            const prefix = path.basename(file.relative, path.extname(file.relative));
            return {
                plugins: [{
                    cleanupIDs: {
                        prefix: prefix + '-',
                        minify: true,
                    },
                }],
            };
        }))
        .pipe(rename(TASK_CONFIG.sprites.rename))
        .pipe(svgstore(TASK_CONFIG.sprites.svgstore))
        .pipe(rename(TASK_CONFIG.sprites.dest_rename))
        .pipe(gulp.dest(dest))
        .pipe(sync.stream());
}

sprites.displayName = 'sprites';
sprites.description = 'Compile svg image sprites.';
sprites.defaultConfig = () => ({
    extensions: ['svg'],
    rename(pathname) {
        const name = pathname.dirname
            .split(pathname.sep)
            .filter((x) => x !== '.');
        name.push(pathname.basename);
        pathname.basename = name.join('-');
    },
    svgstore: {
        inlineSvg: true,
    },
    dest_rename: {},
});
sprites.register = () => {
    gulp.task(sprites);
};
