import autoprefixer from 'autoprefixer';
import gulp from 'gulp';
import csso from 'gulp-csso';
import gulpIf from 'gulp-if';
import postcss from 'gulp-postcss';
import gulpSass from 'gulp-sass';
import sassGlob from 'gulp-sass-glob';
import sourcemaps from 'gulp-sourcemaps';
import dartSass from 'sass';
import handleError from '../lib/handleError';
import projectPath from '../lib/projectPath';
import getSassFunctions from '../sass-functions';
import {sync} from './browser-sync';

const sass = gulpSass(dartSass);

export default function styles() {
    const src = projectPath(PATH_CONFIG.src, PATH_CONFIG.styles.src, '*.{' + TASK_CONFIG.styles.extensions + '}');
    const dest = projectPath(PATH_CONFIG.dest, PATH_CONFIG.styles.dest);

    return gulp.src(src)
        .pipe(gulpIf(GULP_ENV.debug, sourcemaps.init()))
        .pipe(sassGlob())
        .pipe(sass(TASK_CONFIG.styles.sass).on('error', handleError('sass')))
        .pipe(postcss([autoprefixer(TASK_CONFIG.styles.autoprefixer)]))
        .pipe(gulpIf(GULP_ENV.prod, csso(TASK_CONFIG.styles.csso)))
        .pipe(gulpIf(GULP_ENV.debug, sourcemaps.write()))
        .pipe(gulp.dest(dest))
        .pipe(sync.stream());
}
styles.displayName = 'styles';
styles.description = 'Compile sass stylesheets.';
styles.defaultConfig = () => ({
    extensions: ['sass', 'scss', 'css'],
    sass: {
        functions: getSassFunctions(),
        indentWidth: 4,
        outputStyle: 'expanded',
    },
    csso: {
        restructure: true,
        sourceMap: false,
    },
    autoprefixer: {},
});
styles.register = () => {
    gulp.task(styles);
};
