import gulp, {parallel, series} from 'gulp';
import projectPath from '../lib/projectPath';

export default function watch(done) {
    const tasks = ['browser-sync', 'webpack-watch'];

    // If src is an absolute path, gulp.watch() will not detect new or deleted files
    const options = {cwd: projectPath()};
    const baseRegex = new RegExp(`^${options.cwd}/`);
    const sourceDir = projectPath(PATH_CONFIG.src).replace(baseRegex, '');

    return series(...tasks, () => {
        gulp.watch(`${sourceDir}/${PATH_CONFIG.images.src}/**/*`, options, parallel('images'));
        gulp.watch(`${sourceDir}/${PATH_CONFIG.sprites.src}/**/*`, options, parallel('sprites'));
        gulp.watch(`${sourceDir}/${PATH_CONFIG.styles.src}/**/*`, options, parallel('styles'));
        gulp.watch(`${sourceDir}/${PATH_CONFIG.scripts.src}/**/*`, options, parallel('scripts'));

        for (const key of Object.keys(TASK_CONFIG.copy)) {
            const copy = TASK_CONFIG.copy[key];
            gulp.watch(copy.src.replace(baseRegex, '') + '/**/*', options, parallel(`copy:${key}`));
        }
    })(done);
}

watch.displayName = 'watch';
watch.description = 'Watch asset directories for changes and recompile.';
watch.defaultConfig = () => ({});
watch.register = () => {
    gulp.task(watch);
};
