
// Don't expose the Reel class in the global scope.
var ReelFacade;

(function ($) {

$('.reel-field-container[data-content-type="channel"]').each(function() {
    var $field = new Reel($(this));
    $field.init();
});

ReelFacade = Reel;

function Reel ($field) {
    var $addButton = null;
    var $chooseBtn = null;
    var $dataContainer = null;
    var $debugRequests = null;
    var $debugModel = null;
    var $debugData = null;
    var $dropDown = null;
    var $fieldData = null;
    var $fieldPreview = null;
    var $form = null;
    var $iframeContainer = null;
    var $modal = null;
    var $nextBtn = null;
    var $pagination = null;
    var $preview = null;
    var $previewOptions = null;
    var $prevBtn = null;
    var $removeButton = null;
    var $search = null;
    var $tabs = null;
    var $toggleOption = null;
    var $loadingTemplate = null;

    var activeModelName = 'youtube';
    var activeQuery = '';
    var fieldName = null;
    var fieldTemplate = null;
    var globalSettings = null;
    var modalName = null;
    var models = {};
    var rowClassName = $('body').data('eeVersion') ? 'reel-modal-list-service__item' : 'tbl-row';
    var settings = null;

    var init = function () {
        if ($field.length === 0) {
            return;
        }

        models.youtube = new VideoModel({
            search: fetchYouTube,
            embedUrl: 'https://www.youtube.com/embed/{{videoId}}?rel=0&wmode=opaque&hd=1'
        });

        models.vimeo = new VideoModel({
            search: fetchVimeo,
            embedUrl: 'https://player.vimeo.com/video/{{videoId}}?title=0&byline=0&portrait=0'
        });

        findElements();
        parseSettings();
        hidePagination();
        bindClickEvents();

        // Only do this once per field. Each Grid row is considered a field, but they all share the same
        // column ID and use the same modal instance. If we make these calls per row, things go haywire.
        if ($modal.is('[initiated]')) {
            return;
        }
        $modal.attr('initiated', true);

        // Make the first service active
        setActiveTab($tabs.first());

        if (getGlobalSetting('youtube.user') && $modal.find('.toggle-btn--youtube-user').hasClass('on')) {
            models.youtube.set('user', getGlobalSetting('youtube.user'));
        }

        if (getGlobalSetting('youtube.channel') && $modal.find('.toggle-btn--youtube-channel').hasClass('on')) {
            models.youtube.set('channel', getGlobalSetting('youtube.channel'));
        }

        if (getGlobalSetting('vimeo.user') && $modal.find('.toggle-btn--vimeo-user').hasClass('on')) {
            models.vimeo.set('user', getGlobalSetting('vimeo.user'));
        }

        if (getGlobalSetting('youtube.show_on_load') === 'y') {
            models.youtube.search();
        }
        if (getGlobalSetting('vimeo.show_on_load') === 'y') {
            models.vimeo.search();
        }
    };

    var findElements = function () {
        modalName = $field.data('modalName');
        $modal = $('.' + modalName);
        $form = $modal.find('.reel-modal-search-form');
        $search = $modal.find('.reel-modal-search-input');
        $dataContainer = $modal.find('.reel-modal-data-container');
        $debugRequests = $modal.find('.reel-debug-requests');
        $debugModel = $modal.find('.reel-debug-model');
        $debugData = $modal.find('.reel-debug-data');
        $fieldData = $field.find('textarea');
        $fieldPreview = $field.find('.reel-field-preview');
        $iframeContainer = $modal.find('.reel-modal-iframe-container');
        $preview = $modal.find('.reel-modal-preview');
        $previewOptions = $modal.find('.reel-modal-preview-options');
        $pagination = $modal.find('.paginate');
        fieldTemplate = $fieldPreview.html();
        fieldName = $field.data('fieldName');

        if ($loadingTemplate === null && $modal.find('.reel-loading')[0]) {
            $loadingTemplate = $($modal.find('.reel-loading')[0].outerHTML);
            $loadingTemplate.removeClass('hidden');
        }
    };

    /**
     * @param {string} property
     * @param {string} modelName
     * @returns {VideoModel}
     */
    var getModel = function (property, modelName) {
        var model = (modelName) ? getVideoModel(modelName) : models[activeModelName];

        if (property) {
            return model.get(property);
        }

        return model;
    };

    /**
     * @param {string} property
     * @param {string} value
     * @param {string} modelName
     * @returns {VideoModel}
     */
    var setModel = function (property, value, modelName) {
        var model = (modelName) ? getVideoModel(modelName) : models[activeModelName];

        if (property) {
            model.set(property, value);
        }

        updateModelDebug(JSON.stringify(model));

        return model;
    };

    /**
     * @param {string} name
     */
    var setActiveModelName = function (name) {
        activeModelName = name;
    };

    /**
     * @returns {string}
     */
    var getActiveModelName = function () {
        return activeModelName;
    };

    /**
     * @param {string} name
     * @returns {*}
     */
    var getVideoModel = function (name) {
        return models[name];
    };

    var parseSettings = function () {
        settings = $modal.data('settings');
        globalSettings = $modal.data('globalSettings');
    };

    var hidePagination = function () {
        $modal.find('.paginate').hide();
    };

    var bindClickEvents = function () {
        $form.unbind().submit(function (event) {
            event.preventDefault();
            activeQuery = $search.val();

            // Only search the active service's tab
            var model = getModel();
            model.search(activeQuery);
        });

        $chooseBtn = $modal.find('.reel-modal-choose-btn');
        $chooseBtn.unbind().click(chooseVideo);

        $removeButton = $field.find('.reel-field-btn-remove');
        $removeButton.unbind().click(removeVideo);

        $addButton = $field.find('.reel-field-btn-add');
        $addButton.unbind().click(addVideo);

        $tabs = $modal.find('.reel-modal-tabs .js-tab');
        $tabs.unbind().click(clickTab);

        $prevBtn = $pagination.find('.reel-modal-pagination-prev a');
        $prevBtn.unbind().click(prevPage);

        $nextBtn = $pagination.find('.reel-modal-pagination-next a');
        $nextBtn.unbind().click(nextPage);

        $toggleOption = $modal.find('.toggle-btn');
        $toggleOption.unbind().click(toggleOption);

        $dropDown = $modal.find('.reel-modal-filters-dropdown .dropdown__link');
        $dropDown.unbind().click(dropdownOption);
    };

    /**
     * @param {string} key
     * @returns {*}
     */
    var getGlobalSetting = function (key) {
        return key.split('.').reduce(function (globalSettings, i) {
            return globalSettings[i];
        }, globalSettings);
    };

    /**
     * Initiate a search with all available services
     */
    var fetchVideos = function () {
        if (isQueryVideoId()) {
            // @todo remove search filters
        }

        for (var service in models) {
            models[service].search();
        }
    };

    var isQueryVideoId = function () {
        if (activeQuery !== '') {
            var url = activeQuery.match(/^http:\/\/(?:.*?)\.?(youtube|vimeo)\.com\/(watch\?[^#]*v=(\w+)|(\d+))/);

            // See if we have an explicit link to a video URL
            if (url && (url[1] === 'youtube' || url[1] === 'vimeo')) {
                return url[2] || false;
            }
        }

        return false;
    };

    var clickTab = function (event) {
        var $tab = $(event.currentTarget);
        var type = $tab.data('type');

        setActiveTab($tab);
        showResultList(type);

        $form.submit();
        updatePagination();
    };

    var setActiveTab = function ($tab) {
        var type = $tab.data('type');

        setActiveModelName(type);
        updateInstructions(type);

        $tabs.removeClass('active act');
        $tab.addClass('active act');

        $modal.find('.reel-modal-list-' + type).addClass('tab-open');
        $modal.find('.reel-modal-filters').addClass('hidden');
        $modal.find('.reel-modal-filters-' + type).removeClass('hidden');
    };

    var compileExtraQuery = function (type) {
        // If settings are not defined, just return empty string.
        if (!getGlobalSetting(type)) {
            return '';
        }

        var model = getVideoModel(type);
        var params = model.getSearchFilters();

        return '&'+ $.param(params);
    };

    var getResultList = function (type) {
        return $modal.find('.reel-modal-list-'+ type +' ul');
    };

    var showResultList = function (type) {
        $modal.find('.reel-modal-list-service').removeClass('tab-open');
        $modal.find('.reel-modal-list-'+ type).addClass('tab-open');
    };

    var showSpinner = function (type) {
        getResultList(type).html($loadingTemplate);
    };

    var fetchYouTube = function () {
        if (!getGlobalSetting('youtube.api_key')) {
            showError(getResultList('youtube'), 'Please enter your YouTube API key in the Reel module settings page.');
            return;
        }

        showSpinner('youtube');

        var q = getGlobalSetting('load_youtube') +'&q='+ (activeQuery || '') + compileExtraQuery('youtube');

        addRequestDebug(q);

        $.getJSON(q, function (data) {
            if (data.hasOwnProperty('error')) {
                showError(getResultList('youtube'), data.error);
                return;
            }

            setModel('nextPageToken', data.nextPage, 'youtube');
            setModel('prevPageToken', data.prevPage, 'youtube');
            setModel('totalResults', data.totalResults, 'youtube');
            setModel('totalPages', Math.round(data.totalResults / getModel('perPage')), 'youtube');

            getResultList('youtube').html('');
            updatePagination();

            $.each(data.videos, function(i, item) {
                addVideoRow(getResultList('youtube'), item);
            });

            updateInstructions('youtube');
        });
    };

    var fetchVimeo = function () {
        if (!getGlobalSetting('vimeo.client_token')) {
            showError(getResultList('vimeo'), 'Please enter your Vimeo Access Token in the Reel module settings page.');
            return;
        }

        var method = 'videos.getAll';
        var user = getModel('user', 'vimeo');

        if (!user && !activeQuery) {
            showError(getResultList('vimeo'), 'If you are not searching by user, you must enter a search keyword.');
            return;
        }

        // If no query, but we are defaulting to a user's videos on load.
        // YouTube accepts an empty query search value.
        if (!user && activeQuery) {
            method = 'videos.search';
        }

        showSpinner('vimeo');

        var q = getGlobalSetting('load_vimeo') +'&method='+ method + '&query='+ (activeQuery || '') + compileExtraQuery('vimeo');

        addRequestDebug(q);

        $.getJSON(q, function(data) {
            if (!data || data.body === null) {
                showError(getResultList('vimeo'), 'Invalid response. This is most likely an issue with your API credentials, or username.');
                return;
            }

            if (data && data.body && data.body.developer_message) {
                showError(getResultList('vimeo'), data.body.developer_message);
                return;
            }

            if (data.body.error) {
                showError(getResultList('vimeo'), data.body.error);
                return;
            }

            setModel('page', data.body.page, 'vimeo');
            setModel('totalResults', data.body.total, 'vimeo');
            setModel('totalPages', Math.round(data.body.total / getModel('perPage')), 'vimeo');

            getResultList('vimeo').html('');
            updatePagination();

            $.each(data.body.data, function(i, video) {
                var videoId = video.uri.replace('/videos/', '');
                var videoRow = {
                    type: 'vimeo'
                    , description: video.description
                    , author: video.user.name
                    , length: video.duration
                    , length_formatted: new Date(1000 * video.duration).toISOString().substr(11, 8)
                    , title: video.name
                    , video_id: videoId
                    , url: video.link
                };

                // Newer videos have more image options. Older ones appear to only have 3.
                if (video && video.pictures) {
                    if (video.pictures.sizes.length > 3) {
                        videoRow = $.extend(videoRow, {
                            thumbnail_default: video.pictures.sizes[0].link || null
                            , thumbnail_small: video.pictures.sizes[1].link || null
                            , thumbnail_medium: video.pictures.sizes[3].link || null
                            , thumbnail_high: video.pictures.sizes[5].link || null
                            , thumbnail_max: video.pictures.sizes[6].link || null
                        });
                    } else {
                        videoRow = $.extend(videoRow, {
                            thumbnail_default: video.pictures.sizes[0].link || null
                            , thumbnail_medium: video.pictures.sizes[1].link || null
                            , thumbnail_max: video.pictures.sizes[2].link || null
                        });
                    }
                }

                addVideoRow(getResultList('vimeo'), videoRow);
            });

            updateInstructions('vimeo');
        });
    };

    /**
     *
     * @param $target
     * @param message
     */
    var showError = function ($target, message) {
        if ($('body').data('eeVersion')) { // EE6+
            $target.html('<div class="app-notice app-notice--inline app-notice---important"><div class="app-notice__tag"><span class="app-notice__icon"></span></div><div class="app-notice__content"><p class="alert__title">'+ message +'</p></div></div>');
        } else {
            $target.html('<div class="alert inline issue"><p>'+ message +'</p></div>');
        }
    };

    var addVideoRow = function (target, item) {
        var row = $('<li class="media" rel="'+ item.video_id +'"> \
        <div class="'+ rowClassName +'"> \
            <img class="img" src="'+ item.thumbnail_default +'" width="100" /> \
            <div class="bd"> \
                <h3>'+ item.title +'</h3> \
                <p>'+ truncate(item.description) +'</p> \
            </div> \
        </div> \
    </li>');

        row.appendTo(target).click(function(event){
            event.preventDefault();
            updatePreview(item);
        });
    };

    var updatePreview = function (item) {
        var data = JSON.stringify(item);

        $iframeContainer.html(getEmbed(item));
        $dataContainer.val(data);
        $modal.find('.reel-modal-preview-author').html(item.author);
        $modal.find('.reel-modal-preview-length').html(item.length_formatted);
        $modal.find('.reel-modal-preview-description').html(item.description);
        $previewOptions.removeClass('hidden');
        $chooseBtn.show();

        updateDataDebug(data);
    };

    var getEmbed = function (item) {
        var embed = $('<iframe></iframe>');
        var url = getModel('embedUrl').replace('{{videoId}}', item.video_id);

        embed.attr('src', url);

        return embed;
    };

    var addVideo = function (event) {
        event.preventDefault();
        var $btn = $(event.currentTarget);
        $field = $btn.closest('.reel-field-container');

        // When a field's add/change button is clicked, find and rebind all the elements and events
        // so the shared modal (Grid fields etc) is paired up to the current field.
        findElements();
        bindClickEvents();
    };

    var removeVideo = function (event) {
        event.preventDefault();
        var $btn = $(event.currentTarget);
        $field = $btn.closest('.reel-field-container');

        $removeButton.hide();
        $addButton.text('Add Video');
        $fieldData.val('');
        $fieldPreview.hide();
    };

    var chooseVideo = function (event) {
        event.preventDefault();
        $removeButton.show();
        $removeButton.removeClass('disabled');
        $addButton.text('Change Video');

        var video = $dataContainer.val();
        var videoObj = JSON.parse(video);

        for (var key in videoObj) {
            var value = videoObj[key];

            if (typeof value === 'string') {
                // backslash quotes. &quot; would normally be fine, but when this value is placed into the textarea
                // field so the video data can be saved, it renders &quot; as ", so it saves as invalid JSON.
                value = value.replace(/&quot;/g, '\"');
            }

            videoObj[key] = value;
        }

        // Send video data back to the textarea
        $fieldData.val(JSON.stringify(videoObj));

        // Finally update the EE field
        var $template = $(fieldTemplate);
        $template.find('.reel-field-title').html(videoObj.title);
        $template.find('.reel-field-description').html(videoObj.description);
        $template.find('.reel-field-iframe').html(getEmbed(videoObj));
        $template.find('.reel-field-btn-remove').removeClass('disabled');
        $fieldPreview.html($template).show();

        closeModal(event.currentTarget);
    };

    var updatePagination = function () {
        var $pagination = $modal.find('.reel-modal-pagination-'+ getActiveModelName());
        var $activePage = $pagination.find('.act');

        var pageNum = getModel('pageNum');
        var totalPages = getModel('totalPages');

        if (pageNum === 1) {
            $pagination.find('.reel-modal-pagination-prev').hide();
        } else {
            $pagination.find('.reel-modal-pagination-prev').show();
        }

        if (pageNum === totalPages) {
            $pagination.find('.reel-modal-pagination-next').hide();
        } else {
            $pagination.find('.reel-modal-pagination-next').show();
        }

        $modal.find('.paginate').hide(); // Hide the non-active pagination
        $pagination.show();

        $activePage.text(pageNum); // Update our current page
    };

    /**
     * Depending on the service, the page is either an integer or token value.
     * Let the API requests determine what to do with 'page', e.g. is it actually
     * 'offset' (vimeo) or 'pageToken' (youtube)?
     */
    var nextPage = function (event) {
        event.preventDefault();

        var pageNum = getModel('pageNum');
        var totalPages = getModel('totalPages');
        var token = getModel('nextPageToken');
        var newPageNum = (totalPages === pageNum ? pageNum : pageNum + 1);

        if (token) {
            setModel('page', token);
            setModel('pageNum', newPageNum);
        } else {
            setModel('page', newPageNum);
            setModel('pageNum', newPageNum);
        }

        var model = getModel();
        model.search();

        updatePagination();
    };

    var prevPage = function (event) {
        event.preventDefault();

        var pageNum = getModel('pageNum');
        var totalPages = getModel('totalPages');
        var token = getModel('prevPageToken');
        var newPageNum = (pageNum == 1 ? 1 : pageNum - 1);

        if (token) {
            setModel('page', token);
            setModel('pageNum', newPageNum);
        } else {
            setModel('page', newPageNum);
            setModel('pageNum', newPageNum);
        }

        var model = getModel();
        model.search();

        updatePagination();
    };

    var updateInstructions = function (type) {
        var $choose = $modal.find('.reel-make-choice__choose');
        var $find = $modal.find('.reel-make-choice__find');
        var $listContainer = $modal.find('.reel-modal-list-' + type);

        if ($listContainer.find('.media').length === 0) {
            $choose.addClass('hidden');
            $find.removeClass('hidden');
        } else {
            $choose.removeClass('hidden');
            $find.addClass('hidden');
        }
    };

    /**
     * @param element
     */
    var closeModal = function (element) {
        $(element).closest('.modal-wrap').trigger('modal:close');
    };

    var truncate = function(str, length) {
        length = length || 140;
        str = jQuery.trim(str).substring(0, length).split(" ").slice(0, -1).join(" ");

        if (str !== '' && str.length > length) {
            str = str + '...';
        }

        return str;
    };

    var toggleOption = function (event) {
        event.preventDefault();
        var $option = $(event.currentTarget);
        var name = $option.data('optionName');
        var value = $option.data('optionValue');
        var service = $option.data('service');
        var model = getVideoModel(service);

        // It's currently "off", but it'll be changed to "on" when the toggle JS kicks in
        if ($option.hasClass('off')) {
            model.set(name, value);
        } else {
            model.set(name, '');
        }

        model.search();
    };

    var dropdownOption = function (event) {
        event.preventDefault();
        var $option = $(event.currentTarget);
        var $container = $option.closest('.reel-modal-filters-dropdown');
        var $dropdown = $container.find('.dropdown');
        var name = $container.data('optionName');
        var service = $container.data('service');
        var value = $option.data('optionValue');
        var label = $option.text();
        var model = getVideoModel(service);

        $container.find('.faded').text('(' + label + ')');
        $dropdown.removeClass('dropdown--open');

        model.set(name, value);
        model.search();
    };

    var addRequestDebug = function (message) {
        $debugRequests.append('<li>'+ message +'</li>');
    };

    var updateModelDebug = function (model) {
        $debugModel.html('<pre><code>'+ model +'</code></pre>');
    };

    var updateDataDebug = function (data) {
        $debugData.html('<pre><code>'+ data +'</code></pre>');
    };

    return {
        'init': init
    };
}

function VideoModel (options) {
    var properties =  {
        order: null,
        direction: null,
        perPage: 25,
        pageNum: 1,
        page: null,
        user: null,
        channel: null,
        firstPageToken: null,
        nextPageToken: null,
        prevPageToken: null,
        totalResults: 0,
        totalPages: 0,
        embedUrl: null,
        search: null
    };

    $.extend(properties, (options || {}));

    /**
     * @param property
     * @returns {*|null}
     */
    var get = function (property) {
        return properties[property] || null;
    };

    /**
     * @param key
     * @param value
     */
    var set = function (key, value) {
        properties[key] = value;
    };

    /**
     * List of filters that will be compiled into query params
     *
     * @returns {{page: null, perPage: number}}
     */
    var getSearchFilters = function () {
        var filters = {
            'page': properties.page,
            'perPage': properties.perPage
        };

        if (properties.order) {
            filters['order'] = properties.order;
        }

        if (properties.direction) {
            filters['direction'] = properties.direction;
        }

        if (properties.user) {
            filters['user'] = properties.user;
        }

        if (properties.channel) {
            filters['channel'] = properties.channel;
        }

        return filters;
    };

    return {
        'get': get,
        'set': set,
        'search': properties.search,
        '__properties': properties,
        'getSearchFilters': getSearchFilters
    }
}

}(jQuery));
