<?php

namespace BoldMinded\Reel\Service;

class CurlRequest
{
    /**
     * @var array
     */
    private $config = [];

    private $data = [];

    public function __construct(string $url = '', array $data = [], array $headers = [], $callback = NULL)
    {
        if (!function_exists('curl_version')) {
            throw new \Exception(lang('curl_not_installed'));
        }

        $this->config = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => 1,
        ];

        $this->data = $data;

        foreach ($data as $key => $val) {
            if (substr($key, 0, 7) === 'CURLOPT') {
                $this->config[$key] = $val;
            }
        }

        if (!empty($headers)) {
            $this->config[CURLOPT_HTTPHEADER] = $headers;
        }

        if (!empty($callback)) {
            $this->callback = $callback;
        }
    }

    /**
     * @param string $url
     * @param array $data
     * @param array $headers
     * @param \Closure $callback
     * @return Request
     * @throws \Exception
     */
    public function post()
    {
        if (!empty($this->data)) {
            $this->config[CURLOPT_POST] = 1;
            $this->config[CURLOPT_POSTFIELDS] = $this->data;
        }

        return $this->exec();
    }

    private function exec() {
        $curl = curl_init();
        curl_setopt_array($curl, $this->config);
        $data = curl_exec($curl);
        curl_close($curl);

        if (!empty($this->callback)) {
            return call_user_func($this->callback, $data);
        }

        return $data;
    }
}
