<?php

namespace BoldMinded\Reel\Service;

use BoldMinded\Reel\Library\Basee\App;

/**
 * ExpressionEngine Reel Fieldtype Class
 *
 * @package     ExpressionEngine
 * @subpackage  Fieldtypes
 * @category    Reel
 * @author      Brian Litzinger
 * @copyright   Copyright (c) 2017 - Brian Litzinger
 * @link        http://boldminded.com/add-ons/reel
 * @license
 *
 * Copyright (c) 2011, 2012. BoldMinded, LLC
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Brian Litzinger and
 * BoldMinded, LLC) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

class Helper {

    const NAME = 'reel:Helper';

    // Attributes for each video
    public $attributes = [
        'id' => '',
        'url' => '',
        'width' => '360',
        'height' => '240',
        'type' => '',
        'title' => '',
        'description' => '',
        'thumbnail' => '',
        'thumbnail_small' => '',
        'thumbnail_medium' => '',
        'thumbnail_large' => '',
        'thumbnail_max' => '',
        'author' => '',
        'length' => '',
        'embed' => '' // Full iframe embed code
    ];

    // Defaults
    public $default_settings = [
        'settings_global' => [
            'global_width' => '1280',
            'global_height' => '720',
            'global_cache_lifetime' => '3600', // 1 hour
        ],
        'settings_youtube' => [
            'user' => '',
            'channel' => '',
            'api_key' => '',
            'show_on_load' => 'n'
        ],
        'settings_vimeo' => [
            'user' => '',
            'show_on_load' => 'n',
            'client_id' => '',
            'client_secret' => '',
            'client_token' => '',
        ]
    ];

    public $default_settings_type = [
        'settings_global' => [
            'global_width' => 'text',
            'global_height' => 'text',
            'global_cache_lifetime' => 'text',
        ],
        'settings_youtube' => [
            'user' => 'text',
            'channel' => 'text',
            'api_key' => 'text',
            'show_on_load' => 'yes_no'
        ],
        'settings_vimeo' => [
            'user' => 'text',
            'show_on_load' => 'yes_no',
            'client_id' => 'text',
            'client_secret' => 'text',
            'client_token' => 'text',
        ]
    ];

    public $cache = [];
    public $site_url;

    function __construct()
    {
        $this->site_url = ee()->config->slash_item('site_url');
        $this->site_id = ee()->config->item('site_id');
    }

    public function loadAssets()
    {
        if (!isset($this->cache['assetsLoaded'])) {
            ee()->load->library('javascript');

            $modalFile = App::isGteEE6() ? 'modal' : 'modal-legacy';

            ee()->cp->add_to_foot('
                <link rel="stylesheet" href="'. URL_THIRD_THEMES .'reel/styles/'. $modalFile .'.css" />
                <link rel="stylesheet" href="'. URL_THIRD_THEMES .'reel/styles/field.css" />
            ');

            ee()->cp->add_to_foot('
                <script type="text/javascript" src="'. URL_THIRD_THEMES .'reel/scripts/reel.js"></script>
                <script type="text/javascript" src="'. URL_THIRD_THEMES .'reel/scripts/grid.js"></script>
                <script type="text/javascript" src="'. URL_THIRD_THEMES .'reel/scripts/fluid.js"></script>
            ');

            $this->cache['assetsLoaded'] = true;
        }
    }

    public function loadMatrixAssets()
    {
        if (!isset($this->cache['matrixAssetsLoaded'])) {
            ee()->cp->add_to_foot('
                <script type="text/javascript" src="'. URL_THIRD_THEMES .'reel/scripts/matrix2.js"></script>;
            ');

            $this->cache['matrixAssetsLoaded'] = true;
        }
    }

    /**
     * Creates a generic settings row in Grid
     *
     * @param   string  Left hand label for the row
     * @param   string  Content of the row
     * @param   bool    Wide row?
     * @return  string
     */
    private function gridSettingsRow($label, $content, $wide = false)
    {
        $class = $wide ? 'grid_col_setting_label_small_width' : 'grid_col_setting_label_fixed_width';

        return form_label(lang($label), NULL,
            array('class' => $class)
        ).$content;
    }

    /**
     * All URLs should be https, but just in-case account for legacy data
     *
     * @param  string $data
     * @return string
     */
    public function makeUrl($data)
    {
        $data = str_replace('http://', 'https://', $data);

        return $data;
    }

    public function getIframe($data)
    {
        if (isset($data->type)) {
            // Legacy Wyvern Video property name
            $id = isset($data->id) ? $data->id : $data->video_id;

            if ($data->type === 'vimeo') {
                return '<iframe src="https://player.vimeo.com/video/' . $id .'?title=0&byline=0&portrait=0" border="0"></iframe>';
            }

            return '<iframe src="https://www.youtube.com/embed/'. $id .'?rel=0&wmode=opaque" border="0"></iframe>';
        }

        return '';
    }

    /**
     * Get module settings (not individual field settings)
     *
     * @param bool $cleanup
     * @return array|mixed|object
     */
    public function getSettings($cleanup = false)
    {
        // This is called during the install process before this table is created, but during
        // install we don't actually need settings, so just make sure the table exists before
        // proceeding.
        if (!isset($this->cache['settings']) && ee()->db->table_exists('boldminded_settings'))
        {
            $row = ee()->db->select('settings')
                ->where('addon', 'reel')
                ->where('site_id', $this->site_id)
                ->get('boldminded_settings')
                ->row('settings');

            if (empty($row)) {
                $settings = array();
            } else {
                $settings = $this->object_to_array(json_decode($row));
            }

            if (!$settings) {
                $settings = [];
            }

            $this->cache['settings'] = $this->arrayMergeRecursive($this->default_settings, $settings);
        }

        $fileSettings = ee()->config->item('reel');

        if ($fileSettings) {
            $this->cache['settings'] = array_replace($this->cache['settings'], $fileSettings);
        }

        // Cleanup the object so its a little easier to work with without the prefixes
        if ($cleanup && isset($this->cache['settings'])) {
            $settings = [];
            foreach ($this->cache['settings'] as $key => $val) {
                $settings[str_replace('settings_', '', $key)] = $val;
            }

            return $settings;
        }

        return isset($this->cache['settings']) ? $this->cache['settings'] : [];
    }

    /**
     * @param array $array1
     * @param array $array2
     * @return array
     */
    private function arrayMergeRecursive(array $array1, array $array2)
    {
        $merged = $array1;

        foreach ($array2 as $key => & $value) {
            if (is_array($value) && isset($merged[$key]) && is_array($merged[$key])) {
                $merged[$key] = $this->arrayMergeRecursive($merged[$key], $value);
            } else if (is_numeric($key)) {
                if (!in_array($value, $merged)) {
                    $merged[] = $value;
                }
            } else {
                $merged[$key] = $value;
            }
        }

        return $merged;
    }

    public function get_setting_field($name, $value, $group = FALSE)
    {
        $type = 'text';

        if ( isset($this->default_settings[$group][$name]) AND in_array($this->default_settings[$group][$name], array('y', 'n')))
        {
            $type = 'boolean';
            $value = $value ? $value : $this->default_settings[$group][$name];
        }

        switch($type)
        {
            case 'boolean':
                $field = form_checkbox(array(
                    'id' => $name,
                    'name' => 'setting['. $group .']['. $name .']',
                    'checked' => ($value == 'y' ? TRUE : FALSE),
                    'value' => 'y'));
                break;

            default:
                $field = form_input(array(
                    'id' => $name,
                    'name' => 'setting['. $group .']['. $name .']',
                    'style' => 'width: 98%',
                    'value' => $value));
                break;
        }

        return $field;
    }

    public function save_settings($data)
    {
        $data = array(
            'site_id'   => $this->site_id,
            'addon'     => 'reel',
            'settings' => json_encode($data)
        );

        $where = array(
            'site_id'   => $this->site_id,
            'addon'     => 'reel'
        );

        $this->insert_or_update('boldminded_settings', $data, $where, 'id');
    }

    public function get_rte_fields()
    {
        $qry = ee()->db->select('field_id, field_settings, field_type')
            ->where_in('field_type', array('textarea', 'wyvern', 'wygwam'))
            ->where('site_id', ee()->config->item('site_id'))
            ->get('channel_fields');

        $rte_fields = array();

        foreach ($qry->result() as $row)
        {
            $field_settings = unserialize(base64_decode($row->field_settings));

            if ((isset($field_settings['field_enable_rte']) AND $field_settings['field_enable_rte'] == 'y') OR $row->field_type == 'wyvern')
            {
                $rte_fields[] = 'field_id_'.$row->field_id;
            }
        }

        return $rte_fields;
    }

    /*
    @param - string
    @param - array of data to be inserted, key => value pairs
    @param - array of data used to find the row to update, key => value pairs

    _insert_or_update('some_table', array('foo' => 'bar'), array('id' => 1, 'something' => 'another-thing'))

    */
    public function insert_or_update($table, $data, $where, $primary_key = 'entry_id')
    {
        $query = ee()->db->get_where($table, $where);

        // No records were found, so insert
        if ($query->num_rows() == 0)
        {
            ee()->db->insert($table, $data);
            return ee()->db->insert_id();
        }
        // Update existing record
        elseif ($query->num_rows() == 1)
        {
            ee()->db->where($where)->update($table, $data);
            return ee()->db->select($primary_key)->from($table)->where($where)->get()->row($primary_key);
        }
    }

    /**
     * Convert an obj to an array so EE's parse_variables method accepts it.
     *
     * @param object $obj The PHP data object to transform
     * @return array
     */
    public function object_to_array($obj)
    {
        if(is_object($obj)) $obj = (array) $obj;

        if(is_array($obj)) {
            $new = array();
            foreach($obj as $key => $val) {
                $new[$key] = $this->object_to_array($val);
            }
        } else {
            $new = $obj;
        }

        return $new;
    }

    /**
     * Searches a string for an HTML element and grabs the attributes
     *
     * @access  public
     * @param   string
     * @param   regex
     * @return  array($args, $full_tag_match)
     */
    public function get_args($value, $regex)
    {
        preg_match($regex, $value, $matches);
        $raw_tag = isset($matches[0]) ? $matches[0] : '';
        $tag = str_replace(array("\r\n", "\r", "\n", "\t"), " ", $raw_tag);
        return array(trim((preg_match("/\s+.*/", $tag, $matches))) ? $raw_tag : '', $raw_tag);
    }

    /**
     * Return parameters as an array
     *
     * Creates an associative array from a string
     * of parameters: sort="asc" limit="2" etc.
     *
     * @access  public
     * @param   string
     * @return  bool
     */
    public function assign_parameters($str)
    {
        if ($str == "")
            return FALSE;

        // \047 - Single quote octal
        // \042 - Double quote octal

        // I don't know for sure, but I suspect using octals is more reliable than ASCII.
        // I ran into a situation where a quote wasn't being matched until I switched to octal.
        // I have no idea why, so just to be safe I used them here. - Rick

        // matches[0] => attribute and value
        // matches[1] => attribute name
        // matches[2] => single or double quote
        // matches[3] => attribute value
        preg_match_all("/(\S+?)\s*=\s*(\042|\047)([^\\2]*?)\\2/is",  $str, $matches, PREG_SET_ORDER);

        if (count($matches) > 0)
        {
            $result = array();

            foreach($matches as $match)
            {
                $result[$match[1]] = (trim($match[3]) == '') ? $match[3] : trim($match[3]);
            }

            return $result;
        }

        return FALSE;
    }

    public function getSiteIndex()
    {
        $site_index = ee()->config->item('site_index');
        $index_page = ee()->config->item('index_page');

        $index = ($site_index != '') ? $site_index : (($index_page != '') ? $index_page : '');

        return reduce_double_slashes(ee()->config->slash_item('site_url') . $index);
    }

    /**
     * @param $method
     * @param array $params
     * @return null|string
     */
    public function getAction($method, $params = [])
    {
        $action = ee('Model')->get('ee:Action')
            ->filter('class', 'reel')
            ->filter('method', $method)
            ->first();

        if (!$action) {
            return null;
        }

        $actionId = (int) $action->action_id;
        $query = '';

        if (!empty($params)) {
            $query = '&'. http_build_query($params);
        }

        return $this->getSiteIndex() .'?ACT='. $actionId . $query;
    }
}
