<?php

use BoldMinded\Reel\Library\Basee\App;
use BoldMinded\Reel\Library\Basee\License;
use BoldMinded\Reel\Library\Basee\Ping;
use BoldMinded\Reel\Library\Basee\Setting;
use BoldMinded\Reel\Service\Helper;

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * ExpressionEngine Reel Extension Class
 *
 * @package     ExpressionEngine
 * @subpackage  Extensions
 * @category    Reel
 * @author      Brian Litzinger
 * @copyright   Copyright (c) 2017 - Brian Litzinger
 * @link        http://boldminded.com/add-ons/reel
 * @license
 *
 * Copyright (c) 2011, 2012. BoldMinded, LLC
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Brian Litzinger and
 * BoldMinded, LLC) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

class Reel_ext {

    public $settings       = array();
    public $name           = 'Reel';
    public $version        = REEL_VERSION;
    public $description    = '';
    public $settings_exist = 'n';
    public $docs_url       = 'http://boldminded.com/add-ons/reel';
    public $required_by    = array('module');
    public $cache;

    private $iframe_regex = '/<iframe.*?data-video-type="(.*?)".*?><\/iframe>/';
    private $image_regex  = '/<img.*?data-cke-realelement="(.*?)".*?>/';

    /**
     * Constructor
     */
    public function __construct($settings = '') {
        $this->helper = ee(Helper::NAME);
    }

    public function activate_extension() {}
    public function update_extension($current = '') {}
    public function disable_extension() {}

    public function sessions_end($session) {}

    public function core_boot()
    {
        if (REQ === 'CP') {
            $this->validateLicense();
        }
    }

    public function entry_submission_absolute_end($entry_id, $meta, $data)
    {
        $rte_fields = $this->helper->get_rte_fields();

        foreach ($data as $field_name => $field_value)
        {
            $new_data = array();

            // See if its an RTE field
            if (in_array($field_name, $rte_fields))
            {
                preg_match_all($this->image_regex, $field_value, $matches, PREG_SET_ORDER);

                foreach ($matches as $match)
                {
                    $image = $match[0];

                    // Do we have an iframe(s)? If so find them, and grab the attributes.
                    list($args, $image) = $this->helper->get_args($image, $this->image_regex);
                    $params = $this->helper->assign_parameters($args);

                    if ($params)
                    {
                        $iframe = rawurldecode($params['data-cke-realelement']);

                        preg_match($this->iframe_regex, $iframe, $matches);

                        // If false/blank then we don't need to add these attributes as the iframe element already has them.
                        if ( ! isset($matches[1]) OR $matches[1] == '')
                        {
                            // Make sure our attributes remain on the iframe tag. Save the thumb too as it will avoid an extra API lookup in publish_form_entry_data
                            $attr = 'data-video-thumbnail="'. $params['src'] .'" data-video-type="'. $params['data-video-type'] .'" data-video-id="'. $params['data-video-id'] .'" data-query="'. $params['data-query'] .'"';

                            // Add attr
                            $iframe = preg_replace('/(src=".*?")/', "$1 $attr", $iframe);
                        }

                        // Swap out the image entirely with our iframe
                        $field_value = str_replace($image, $iframe, $field_value);
                    }
                }

                // Because we can't manipulate the $_POST data prior to EE using it
                $this->helper->insert_or_update('channel_data', array($field_name => $field_value), array(
                    'entry_id' => $entry_id
                ));
            }
        }
    }

    public function publish_form_entry_data($data)
    {
        if (ee()->extensions->last_call)
        {
            $data = ee()->extensions->last_call;
        }

        $rte_fields = $this->helper->get_rte_fields();

        foreach ($data as $field_name => $field_value)
        {
            // See if its an RTE field
            if (in_array($field_name, $rte_fields))
            {
                preg_match_all($this->iframe_regex, $field_value, $matches, PREG_SET_ORDER);

                foreach ($matches as $match)
                {
                    $iframe = $match[0];

                    // Do we have an iframe(s)? If so find them, and grab the attributes.
                    list($args, $iframe) = $this->helper->get_args($iframe, $this->iframe_regex);
                    $params = $this->helper->assign_parameters($args);

                    if ($params)
                    {
                        // @todo - need method to lookup thumbnails
                        $params['src'] = isset($params['data-video-thumbnail']) ? $params['data-video-thumbnail'] : '';
                        $params['data-cke-realelement'] = rawurlencode($iframe);

                        $image = '<img ';

                        foreach ($params as $pk => $pv)
                        {
                            $image .= $pk .'="'. $pv .'" ';
                        }

                        $image = trim(substr($image, 0, -1));

                        $image .= '>';

                        // Swap out the iframe entirely with our image
                        $field_value = str_replace($iframe, $image, $field_value);
                    }
                }

                $data[$field_name] = $field_value;
            }
        }

        return $data;
    }

    public function wygwam_config($config, $settings)
    {
        // If another extension shares the same hook, we need to get the latest and greatest config
        if(!empty(ee()->extensions->last_call)) {
            $config = ee()->extensions->last_call;
        }

        if (isset($config['extraPlugins']) AND strstr($config['extraPlugins'], 'video') !== FALSE)
        {
            $config['toolbar'][]  = array('Video');
        }

        return $config;
    }

    private function validateLicense()
    {
        if (App::isInstallingOrUninstallingAddon('reel')) {
            return;
        }

        $ping = new Ping('reel_last_ping', 2400);

        /** @var Setting $setting */
        $setting = ee('reel:Setting');
        $licenseKey = $setting->get('license');

        if ($ping->shouldPing() || !$licenseKey) {
            $ping->updateLastPing();

            $license = new License('https://license.boldminded.com');
            $response = $license->checkLicense([
                'payload' => base64_encode(json_encode([
                    'a'   => 'Reel',
                    'api' => '1',
                    'b'   => REEL_BUILD_VERSION,
                    'd'   => ee()->config->item('base_url'),
                    'e'   => APP_VER,
                    'i'   => 1433,
                    'l'   => $licenseKey,
                    'p'   => phpversion(),
                    's'   => ee()->config->item('site_id'),
                    'v'   => REEL_VERSION,
                ]))
            ]);

            if (
                defined('BASE') &&
                ($response !== null && isset($response['status'])) &&
                (!$licenseKey || $response['status'] === 'invalid') &&
                (!empty(ee()->uri->rsegments) && end(ee()->uri->rsegments) !== 'license')
            ) {
                ee('CP/Alert')
                    ->makeInline('shared-form')
                    ->asWarning()
                    ->withTitle('License is invalid.')
                    ->addToBody('Please enter a valid license. Your license is available at boldminded.com, or expressionengine.com. If you purchased from expressionengine.com, be sure to visit boldminded.com/claim to add the license to your account.')
                    ->defer();

                ee()->functions->redirect(
                    ee('CP/URL')->make('addons/settings/reel/license')->compile()
                );
            }
        }
    }
}
