<?php

use BoldMinded\Reel\Library\Basee\App;
use BoldMinded\Reel\Service\Helper;

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * ExpressionEngine Reel Fieldtype Class
 *
 * @package     ExpressionEngine
 * @subpackage  Fieldtypes
 * @category    Reel
 * @author      Brian Litzinger
 * @copyright   Copyright (c) 2017 - Brian Litzinger
 * @link        http://boldminded.com/add-ons/reel
 * @license
 *
 * Copyright (c) 2011, 2012. BoldMinded, LLC
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Brian Litzinger and
 * BoldMinded, LLC) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

class Reel_ft extends EE_Fieldtype {

    public $settings = [];
    public $has_array_data = false;
    public $entry_manager_compatible = true;
    public $info = [
        'name' => 'Reel',
        'version' => REEL_VERSION
    ];

    private $site_id;
    private $cache = [];

    /**
     * @var Helper
     */
    private $helper;

    /**
     * @var string Front-edit window size
     */
    public $size = 'large';

    /**
     * Constructor
     *
     * @access  public
     */
    public function __construct()
    {
        parent::__construct();

        $this->site_id = ee()->config->item('site_id');

        // Create cache
        if (! isset(ee()->session->cache['reel'])) {
            ee()->session->cache['reel'] = [];
        }
        $this->cache =& ee()->session->cache['reel'];
        $this->helper = ee(Helper::NAME);
        $this->settings = $this->helper->getSettings();
    }

    public function validate($data)
    {
        return true;
    }

    public function update()
    {
        return true;
    }

    /**
     * @param $name
     * @return bool
     */
    public function accepts_content_type($name)
    {
        $acceptedTypes = [
            'channel',
            'grid',
            'blocks/1',
            'fluid_field'
        ];

        return in_array($name, $acceptedTypes);
    }

    /**
     * @param $array
     * @return array
     */
    private function getSettingsFromArray($array)
    {
        // Check for legacy data
        if (isset($array['wyvern_video'])) {
            $settings = $array['wyvern_video'];
        } else {
            $settings = isset($array['reel']) ? $array['reel'] : [];
        }

        return $settings;
    }

    /**
     * Display content in EE6+ Entry Manager.
     *
     * @param mixed $data
     * @param int $fieldId
     * @param int $entry
     * @return string
     */
    public function renderTableCell($data, $fieldId, $entry)
    {
        if (!$data) {
            return '';
        }

        // Decode the JSON data
        $data = $this->pre_process($data);

        return ee('View')->make('reel:thumb')->render([
            'url' => $this->replace_thumbnail_medium($data),
            'title' => $this->replace_title($data),
            'length' => $this->replace_length($data, [
                'format' => '%H:%i:%s'
            ]),
        ]);
    }

    /**
     * @return array
     */
    public function getTableColumnConfig()
    {
        return [
            'encode' => false
        ];
    }

    /**
     * Default field display
     *
     * @param $data
     * @param bool $cellName
     * @param bool $colId
     * @return string
     */
    public function display_field($data, $cellName = false, $colId = false)
    {
        $this->helper->loadAssets();
        $settings = $this->getSettingsFromArray($this->settings);
        $globalSettings = $this->helper->getSettings(true);
        $globalSettings['load_vimeo'] = $this->helper->getAction('load_vimeo');
        $globalSettings['load_youtube'] = $this->helper->getAction('load_youtube');

        $disabled = 'disabled';
        $btnAddText = 'Add Video';
        $video = null;

        // Make sure we have data, and a video ID first.
        if ($data && !is_array($data)) {
            $video = json_decode(html_entity_decode($data, ENT_COMPAT, 'UTF-8'));

            // Unsure why this is needed, but in some cases the decoding above did not work, even though that line
            // of code has been unchanged for years. https://boldminded.com/support/ticket/1827
            if (!$video) {
                $video = json_decode($data);
            }

            // Normalize old Wyvern Video field data
            if (isset($video->id)) {
                $video->video_id = $video->id;
            }

            if (isset($video->video_id) && $video->video_id != '') {
                $disabled = '';
                $btnAddText = 'Change Video';
            }
        }

        // Normal field or Grid/Matrix cell?
        $fieldName = $cellName ? $cellName : $this->field_name;
        $fieldNameConfig = $colId ? 'col_id_'.$colId : $fieldName;
        // Clean it up
        $fieldNameClean = str_replace(['[',']'], '', $fieldName);
        if ($fieldNameClean === '{DEFAULT}') {
            $fieldModalName = 'reel-modal-matrix';
        } else {
            $fieldModalName = 'reel-modal-'.$fieldNameClean;
        }

        $modalVars = [
            'fieldName' => $fieldNameClean,
            'fieldModalName' => $fieldModalName,
            'settings' => $settings,
            'globalSettings' => $globalSettings,
            'settingsEncoded' => htmlentities(json_encode($settings), ENT_QUOTES, 'UTF-8'),
            'globalSettingsEncoded' => htmlentities(json_encode($globalSettings), ENT_QUOTES, 'UTF-8'),
            'isModernEE' => App::isGteEE6(),
            'debug' => ee()->input->get('reel_debug'),
        ];

        if (App::isGteEE6()) {
            $modalHtml = ee('View')->make('reel:modal')->render($modalVars);
        } else {
            $modalHtml = ee('View')->make('reel:modal-legacy')->render($modalVars);
        }

        ee('CP/Modal')->addModal($fieldModalName, $modalHtml);

        $fieldVars = [
            'fieldName' => $fieldName,
            'fieldNameClean' => $fieldNameClean,
            'fieldNameConfig' => $fieldNameConfig,
            'fieldModalName' => $fieldModalName,
            'fieldAttributes' => $this->helper->attributes,
            'iframe' => $this->helper->getIframe($video),
            'btnAddText' => $btnAddText,
            'data' => $data,
            'video' => $video,
            'disabled' => $disabled,
            'settings' => $settings,
            'contentType' => $this->content_type,
            'isModernEE' => App::isGteEE6(),
        ];

        return ee('View')->make('reel:field')->render($fieldVars);
    }

    /**
     * Matrix cell display
     *
     * @param $data
     * @return mixed
     */
    public function display_cell($data)
    {
        $r['class'] = 'reel-matrix';
        $r['data'] = $this->display_field($data, $this->cell_name, $this->col_id);

        $this->helper->loadMatrixAssets();

        return $r;
    }

    /**
     * Low Variables Fieldtype Display
     *
     * @param $data
     * @return string
     */
    public function display_var_field($data)
    {
        return $this->display_field($data, 'var['. $this->var_id .']');
    }

    /**
     * @param $data
     * @return string
     */
    public function grid_display_field($data)
    {
        return $this->display_field($data);
    }

    /**
     * Native EE save
     *
     * @param $data
     * @return mixed
     */
    public function save($data)
    {
        return $data;
    }

    /**
     * @param $data
     * @return string
     */
    public function block_save($data)
    {
        $columnId = $this->id;
        $columnName = $this->field_name;
        $fieldId = $this->settings['grid_field_id'];
        $rowId = (int) $this->settings['grid_row_id'];
        $blockId = isset($this->settings['blocks_block_id']) ? $this->settings['blocks_block_id'] : null;
        $blockPostId = $blockId;

        if (!isset($_POST['field_id_'.$fieldId])) {
            return '';
        }

        $isNewBlock = preg_match('/^blocks_new_block_/', $blockId);

        if (
            !isset($_POST['field_id_'.$fieldId]['blocks_block_id_'.$rowId]) &&
            !$isNewBlock
        ) {
            return '';
        }

        if (!$isNewBlock) {
            $blockPostId = 'blocks_block_id_'.$rowId;
        }

        $postDataColumnValues = $_POST['field_id_'.$fieldId][$blockPostId]['values'];

        foreach ($postDataColumnValues as $index => $columnValue) {
            if (substr($index, 0, strlen($columnName)) === $columnName) {
                return $columnValue;
            }
        }

        return $data;
    }


    /**
     * Save for Matrix
     *
     * @param $data
     * @return mixed
     */
    public function save_cell($data)
    {
        return $this->save($data);
    }

    /**
     * Save Low Variables field
     *
     * @param $data
     * @return mixed
     */
    public function save_var_field($data)
    {
        return $this->save($data);
    }

    /**
     * Display native EE field settings
     *
     * @param $data
     * @return array
     */
    public function display_settings($data)
    {
        return $this->getFieldSettings($data);
    }

    /**
     * Display Matrix cell settings
     *
     * @param $data
     * @return array
     */
    public function display_cell_settings($data)
    {
        return $this->getFieldSettings($data, true);
    }

    /**
     * Display Low Variables field settings
     *
     * @param $data
     * @return array
     */
    public function display_var_settings($data)
    {
        return $this->getFieldSettings($data);
    }

    /**
     * Display Grid field settings
     *
     * @param $data
     * @return array
     */
    public function grid_display_settings($data)
    {
        return $this->getFieldSettings($data);
    }

    /*
     * Save individual field settings
     */
    public function save_settings($settings)
    {
        $data = ee()->input->post('reel');
        $globalSettings = $this->helper->getSettings();

        $saveSettings = [];

        $saveSettings['reel']['show_details'] = isset($data['show_details']) ? $data['show_details'] : 'no';
        $saveSettings['reel']['field_width'] = isset($data['field_width']) ? $data['field_width'] : $globalSettings['settings_global']['global_width'];
        $saveSettings['reel']['field_height'] = isset($data['field_height']) ? $data['field_height'] : $globalSettings['settings_global']['global_height'];
        $saveSettings['reel']['allow_resize'] = isset($data['allow_resize']) ? $data['allow_resize'] : 'no';

        $saveSettings['field_fmt'] = 'none';
        $saveSettings['field_show_fmt'] = 'n';
        $saveSettings['field_wide'] = true;

        return $saveSettings;
    }

    /**
     * @param $data
     * @param bool $isMatrix
     * @return array
     */
    private function getFieldSettings($data, $isMatrix = false)
    {
        $settings = $this->getSettingsFromArray($data);

        $sections = [
            [
                'title' => 'show_details',
                'desc' => lang('show_details_desc'),
                'fields' => [
                    'reel[show_details]' => [
                        'type' => 'yes_no',
                        'value' => isset($settings['show_details']) ? $settings['show_details'] : 'n'
                    ]
                ]
            ],
        ];

        if (in_array($this->content_type(), ['grid', 'blocks', 'blocks/1'])) {
            return ['field_options' => $sections];
        }

        // Ugh, Matrix couldn't just support the new fieldtype settings?
        if ($isMatrix) {
            $matrixSettings = [];
            foreach ($sections as $section) {
                foreach ($section['fields'] as $fieldName => $fieldOptions) {
                    $vars = [
                        'grid' => false,
                        'field' => array_merge([
                            'name' => $fieldName,
                        ], $fieldOptions)
                    ];
                    $matrixSettings[] = [
                        lang($section['title']),
                        ee('View')->make('ee:_shared/form/field')->render($vars)
                    ];
                }
            }

            return $matrixSettings;
        }

        return ['field_options_reel' => [
            'label' => 'field_options',
            'group' => 'reel',
            'settings' => $sections
        ]];
    }

    /**
     * Save Matrix Cell Settings
     */
    public function save_cell_settings($settings)
    {
        $global_settings = $this->helper->getSettings();

        return [
            'reel' => [
                'show_details'  => isset($settings['reel']['show_details']) ? $settings['reel']['show_details'] : 'no',
                'field_width'  => isset($settings['reel']['field_width']) ? $settings['reel']['field_width'] : $global_settings['settings_global']['global_width'],
                'field_height'  => isset($settings['reel']['field_height']) ? $settings['reel']['field_height'] : $global_settings['settings_global']['global_height'],
                'allow_resize'  => isset($settings['reel']['allow_resize']) ? $settings['reel']['allow_resize'] : 'no'
            ]
        ];
    }

    /**
     * Save Low Variables Settings
     */
    public function save_var_settings($settings)
    {
        return $this->save_settings($settings);
    }

    /**
     * Just need an empty method here since we are not doing anything special.
     */
    public function grid_save_settings()
    {
        return;
    }

    /**
     * @param $data
     * @return mixed
     */
    public function pre_process($data)
    {
        if ($data) {
            $data = json_decode(html_entity_decode($data, ENT_COMPAT, 'UTF-8'));
        }

        return $data;
    }

    /**
     * @param $data
     * @param array $params
     * @param bool $tagdata
     * @return string
     */
    public function replace_tag($data, $params = [], $tagdata = false)
    {
        // Normalize old Wyvern Video field data
        if (isset($data->id)) {
            $data->video_id = $data->id;
        }

        // If the embed property exists, its a Wyvern Video upgrade
        if (isset($data->embed)) {
            return $this->helper->makeUrl(urldecode($data->embed));
        }

        return $this->helper->getIframe($data);
    }

    /**
     * Low Variables tag parsing
     *
     * @param $data
     * @param $params
     * @param $tagdata
     * @return string
     */
    public function display_var_tag($data, $params, $tagdata)
    {
        $data = $this->pre_process($data);

        // We have a pair... (and hopefuly the user actually used the :pair method)
        if ($tagdata) {
            if (isset($params['variable_prefix'])) {
                $prefix = $params['variable_prefix'];
                $prefixed_data = [];

                foreach ($data as $k => $v) {
                    $prefixed_data[$prefix.$k] = $v;
                }

                $data = $prefixed_data;
            }

            // This should always be present, but just incase
            if (isset($data['embed'])) {
                $data['embed'] = $this->helper->makeUrl(urldecode($data->embed));
            }

            return ee()->TMPL->parse_variables($tagdata, [$data]);
        } else {
            // See if the user has requested a specific video attribute
            if (isset($params['attribute']) && isset($data->$params['attribute'])) {
                return $data->$params['attribute'];
            } else {
                return $this->helper->makeUrl(urldecode($data->embed));
            }
        }
    }

    public function replace_url($data, $params = [], $tagdata = false)
    {
        return isset($data->url) ? $this->helper->makeUrl($data->url) : '';
    }

    public function replace_title($data, $params = [], $tagdata = false)
    {
        return isset($data->title) ? $data->title : '';
    }

    public function replace_title_attr_safe($data, $params = [], $tagdata = false)
    {
        return isset($data->title) ? htmlentities($data->title, ENT_QUOTES) : '';
    }

    public function replace_description($data, $params = [], $tagdata = false)
    {
        return isset($data->description) ? $data->description : '';
    }

    public function replace_width($data, $params = [], $tagdata = false)
    {
        return isset($data->width) ? $data->width : '';
    }

    public function replace_height($data, $params = [], $tagdata = false)
    {
        return isset($data->height) ? $data->height : '';
    }

    public function replace_id($data, $params = [], $tagdata = false)
    {
        // Legacy Wyvern Video property name
        if (isset($data->id)) {
            return $data->id;
        }

        if (isset($data->video_id)) {
            return $data->video_id;
        }

        return '';
    }

    public function replace_author($data, $params = [], $tagdata = false)
    {
        return isset($data->author) ? $data->author : '';
    }

    public function replace_length($data, $params = [], $tagdata = false)
    {
        if (isset($data->length)) {
            $stamp = strtotime($data->length);

            if (isset($params['format'])) {
                $format = str_replace('%', '', $params['format']);

                return date($format, $stamp);
            }

            return $stamp;
        }

        return '';
    }

    public function replace_thumbnail($data, $params = [], $tagdata = false)
    {
        return isset($data->thumbnail_default) ? $data->thumbnail_default : '';
    }

    public function replace_thumbnail_small($data, $params = [], $tagdata = false)
    {
        // YouTube
        if (isset($data->thumbnail_medium)) {
            return $data->thumbnail_medium;
        }

        return isset($data->thumbnail_small) ? $data->thumbnail_small : '';
    }

    public function replace_thumbnail_medium($data, $params = [], $tagdata = false)
    {
        // YouTube
        if (isset($data->thumbnail_high)) {
            return $data->thumbnail_high;
        }

        return isset($data->thumbnail_medium) ? $data->thumbnail_medium : '';
    }

    public function replace_thumbnail_large($data, $params = [], $tagdata = false)
    {
        // YouTube
        if (isset($data->thumbnail_standard)) {
            return $data->thumbnail_standard;
        }

        return isset($data->thumbnail_high) ? $data->thumbnail_high : '';
    }

    public function replace_thumbnail_max($data, $params = [], $tagdata = false)
    {
        return isset($data->thumbnail_max) ? $data->thumbnail_max : '';
    }

    public function replace_type($data, $params = [], $tagdata = false)
    {
        return isset($data->type) ? $data->type : '';
    }
}
