<?php

use BoldMinded\Reel\Library\Basee\Setting;
use BoldMinded\Reel\Service\Helper;

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * ExpressionEngine Reel Module Class
 *
 * @package     ExpressionEngine
 * @subpackage  Modules
 * @category    Reel
 * @author      Brian Litzinger
 * @copyright   Copyright (c) 2017 - Brian Litzinger
 * @link        http://boldminded.com/add-ons/reel
 * @license
 *
 * Copyright (c) 2011, 2012. BoldMinded, LLC
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Brian Litzinger and
 * BoldMinded, LLC) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

class Reel_mcp {

    /**
     * @var Helper
     */
    private $helper;

    function __construct()
    {
        $this->helper = ee(Helper::NAME);

        /** @var Sidebar $sidebar */
        $sidebar = ee('CP/Sidebar')->make();
        $sidebar->addHeader(lang('reel_settings'), ee('CP/URL')->make('addons/settings/reel'));
        $sidebar->addHeader(lang('reel_license'), ee('CP/URL')->make('addons/settings/reel/license'));
    }

    public function index()
    {
        if (ee()->config->item('reel')) {
            /** @var \EllisLab\ExpressionEngine\Service\View\View $view */
            $view = ee('View')->make('reel:no');
            return $view->render();
        }

        ee()->load->library('table');
        ee()->load->helper('path');
        ee()->load->helper('form');

        $vars['hidden'] = null;
        $vars['base_url'] = ee('CP/URL')->make('addons/settings/reel/save_settings');
        $vars['save_btn_text'] = lang('btn_save_settings');
        $vars['save_btn_text_working'] = lang('btn_saving');
        $vars['cp_page_title'] = 'Reel Configuration';

        $vars['fields'] = $this->helper->default_settings;
        $settings = $this->helper->getSettings();
        $sections = [];

        if ($settings) {
            foreach ($settings as $group => $data) {
                foreach($data as $key => $value) {
                    $desc = lang($group.'_'.$key.'_desc');
                    $field = [
                        'title' => lang($group.'_'.$key),
                        'fields' => [
                            'setting['. $group .']['. $key .']' => [
                                'type' => $this->helper->default_settings_type[$group][$key],
                                'value' => $value,
                            ]
                        ]
                    ];

                    if ($desc != '') {
                        $field['desc'] = $desc;
                    }

                    $sections[lang('reel_' . $group)][] = $field;
                }
            }
        }

        $vars['sections'] = $sections;

        /** @var \EllisLab\ExpressionEngine\Service\View\View $view */
        $view = ee('View')->make('reel:index');
        return $view->render($vars);
    }

    public function save_settings()
    {
        $data = ee()->input->post('setting');
        $this->helper->save_settings($data);

        $alert = ee('CP/Alert');
        $alert
            ->makeInline('shared-form')
            ->asSuccess()
            ->withTitle(lang('settings_saved'))
            ->defer();

        ee()->functions->redirect(ee('CP/URL')->make('addons/settings/reel'));
    }

    public function license()
    {
        /** @var Setting $setting */
        $setting = ee('reel:Setting');

        if ($license = ee('Request')->post('license')) {
            $setting->save([
                'license' => $license,
            ]);

            ee('CP/Alert')
                ->makeInline('shared-form')
                ->asSuccess()
                ->withTitle('Success')
                ->addToBody('License updated!')
                ->now();
        }

        $sections = [
            [
                [
                    'title' => 'reel_license_name',
                    'desc' => lang('reel_license_desc'),
                    'fields' => [
                        'license' => [
                            'required' => true,
                            'type' => 'text',
                            'value' => $setting->get('license'),
                        ]
                    ]
                ],
            ],
        ];

        $vars['sections'] = $sections;
        $vars['base_url'] = ee('CP/URL')->make('addons/settings/reel/license');
        $vars['save_btn_text'] = lang('save');
        $vars['save_btn_text_working'] = lang('saving');
        $vars['cp_page_title'] = '';

        /** @var \EllisLab\ExpressionEngine\Service\View\View $view */
        $view = ee('View')->make('ee:_shared/form');
        return $view->render($vars);
    }
}
