<?php

use BoldMinded\Reel\Library\Basee\Setting;
use BoldMinded\Reel\Library\Basee\Update\AbstractUpdate;
use BoldMinded\Reel\Service\Helper;

class Update_1_00_00 extends AbstractUpdate
{
    public function doUpdate()
    {
        $db = ee('db');
        $prefix = $db->dbprefix;

        // Module data
        $data = [
            'module_name' => 'Reel',
            'module_version' => REEL_VERSION,
            'has_cp_backend' => 'y',
            'has_publish_fields' => 'n'
        ];

        $db->insert('modules', $data);

        ee()->load->dbforge();

        if (!$db->table_exists('boldminded_settings')) {
            ee()->dbforge->add_field([
                'id'        => ['type' => 'int', 'constraint' => 10, 'unsigned' => true, 'auto_increment' => true],
                'site_id'   => ['type' => 'int', 'constraint' => 4, 'unsigned' => true],
                'addon'     => ['type' => 'varchar', 'constraint' => 256],
                'settings'  => ['type' => 'text'],
            ]);

            ee()->dbforge->add_key('id', true);
            ee()->dbforge->create_table('boldminded_settings');

            $db->insert('boldminded_settings', [
                'site_id'   => ee()->config->item('site_id'),
                'addon'     => 'reel',
                'settings'  => json_encode([])
            ]);
        }
        elseif ($db->table_exists('boldminded_settings'))
        {
            $exists = $db->where('addon', 'reel')->count_all_results('boldminded_settings');

            if ($exists == 0) {
                /** @var Helper $helper */
                $helper = ee(Helper::NAME);

                $db->insert('boldminded_settings', [
                    'addon'     => 'reel',
                    'settings'  => json_encode($helper->default_settings)
                ]);
            }
        }

        // Insert our Action(s)
        /** @var CI_DB_result $query */
        $query = $db->get_where('actions', ['class' => 'reel']);

        if($query->num_rows() == 0) {
            $data = array(
                'class' => 'reel',
                'method' => 'load_vimeo'
            );

            $db->insert('actions', $data);

            $data = array(
                'class' => 'reel',
                'method' => 'load_youtube'
            );

            $db->insert('actions', $data);
        }

        // Delete old hooks
        $db->where('class', 'Reel_ext')->delete('extensions');

        // Add new hooks
        $ext_template = [
            'class'    => 'Reel_ext',
            'settings' => '',
            'priority' => 5,
            'version'  => REEL_VERSION,
            'enabled'  => 'y'
        ];

        $extensions = [
            ['hook'=>'sessions_end', 'method'=>'sessions_end'],
            ['hook'=>'publish_form_entry_data', 'method'=>'publish_form_entry_data'],
            ['hook'=>'entry_submission_absolute_end', 'method'=>'entry_submission_absolute_end'],
            ['hook'=>'wygwam_config', 'method'=>'wygwam_config']
        ];

        foreach($extensions as $extension) {
            $ext = array_merge($ext_template, $extension);
            $db->insert('exp_extensions', $ext);
        }

        // Migrate from old Wyvern Video fields
        $db->query("update `". $prefix ."channel_fields` set field_type = 'reel' where field_type = 'wyvern_video'");
        $db->query("delete from `". $prefix ."modules` where module_name = 'Wyvern_video'");
        $db->query("delete from `". $prefix ."fieldtypes` where `name` = 'wyvern_video'");
        $db->query("delete from `". $prefix ."extensions` where class = 'Wyvern_video_ext'");

        if ($db->table_exists('grid_columns')) {
            $db->query("update `". $prefix ."grid_columns` set col_type = 'reel' where col_type = 'wyvern_video'");
        }

        if ($db->table_exists('blocks_atomdefinition')) {
            $db->query("update `". $prefix ."blocks_atomdefinition` set `type` = 'reel' where `type` = 'wyvern_video'");
        }

        if ($db->table_exists('matrix_cols')) {
            $db->query("update `". $prefix ."matrix_cols` set col_type = 'reel' where col_type = 'wyvern_video'");
        }

        /** @var Setting $setting */
        $setting = ee('reel:Setting');
        $setting->createTable();
        $setting->save([
            'installed_date' => time(),
            'installed_version' => REEL_VERSION,
            'installed_build' => REEL_BUILD_VERSION,
        ]);
    }
}
