<?php

class Datagrab_wm_address extends AbstractFieldType
{
    /** @var array */
    private $fields = [
        'name'        => 'Name',
        'street'      => 'Street Address',
        'street_2'    => 'Street Address Line 2',
        'city'        => 'City',
        'region'      => 'State/Province',
        'postal_code' => 'ZIP/Postal Code',
        'country'     => 'Country',
    ];

    /**
     * @param string $fieldName
     * @return array
     */
    public function register_setting(string $fieldName)
    {
        $settings = [];
        foreach ($this->fields as $name => $label) {
            $settings[] = $fieldName . '_wm_address_' . $name;
        }

        return $settings;
    }

    public function display_configuration(Datagrab_model $DG, string $fieldName, string $fieldLabel, string $fieldType, bool $fieldRequired = false, array $data = []): array
    {
        $config = [];
        $config['label'] = form_label($fieldLabel);

        if ($fieldRequired) {
            $config['label'] .= ' <span class="datagrab_required">*</span>';
        }

        $config['label'] .= '<div class="datagrab_subtext">' . $fieldType . '</div>';

        $config['value'] = '';
        $config['value'] .= form_hidden($fieldName, '1');

        foreach ($this->fields as $name => $label) {
            $inputLabel = form_label($label);
            $inputField = $this->build_dropdown($fieldName . '_wm_address_' . $name, $data);

            $config['value'] .= sprintf('<p>%s:<br>%s</p>', $inputLabel, $inputField);
        }

        return $config;
    }

    public function prepare_post_data(Datagrab_model $DG, array $item = [], int $fieldId = 0, string $fieldName = '', array &$data = [], int $updateEntryId = 0)
    {
        $data['field_id_' . $fieldId] = [];

        foreach ($this->fields as $name => $label) {
            $subFieldName = $fieldName . '_wm_address_' . $name;

            if ($DG->settings['cf'][$subFieldName] !== '') {
                $data['field_id_' . $fieldId][$name] = $DG->dataType->get_item($item, $DG->settings['cf'][$subFieldName]);
            }
        }
    }

    private function build_dropdown(string $fieldName, array $data): string
    {
        $fieldValue = '';

        if (isset($data['default_settings']['cf'][$fieldName])) {
            $fieldValue = $data['default_settings']['cf'][$fieldName];
        }

        return form_dropdown($fieldName, $data['data_fields'], $fieldValue);
    }

}
