<?php

namespace WarbleMedia\AssetManifest\Service;

class Manifest
{
    /** @var array */
    private $manifest;

    /**
     * Manifest constructor.
     *
     * @param array $manifest
     */
    public function __construct(array $manifest = [])
    {
        $this->manifest = $manifest;
    }

    /**
     * Create an asset manifest from a file path.
     *
     * @param string $filename
     * @return \WarbleMedia\AssetManifest\Service\Manifest
     */
    public static function createFromFile($filename)
    {
        if (!is_file($filename) && !is_readable($filename)) {
            //user_error('Manifest file does not exist or is not readable.', E_USER_NOTICE);
            return new self();
        }

        $contents = file_get_contents($filename);
        $contents = json_decode($contents, true);

        if ($contents === null && json_last_error() !== JSON_ERROR_NONE) {
            user_error('Manifest file does not contain valid json.', E_USER_NOTICE);
            return new self();
        }

        return new self($contents);
    }

    /**
     * Revisioned path to retrieve.
     *
     * @param string $path
     * @return string
     */
    public function getPath($path)
    {
        foreach ($this->manifest as $original => $revisioned) {
            if (preg_match("~^(.*){$original}$~", $path, $matches)) {
                return $matches[1] . $revisioned;
            }
        }

        return $path;
    }
}
