<?php

class Wm_json_templates_ext
{
    /** @var string */
    public $name;

    /** @var string */
    public $version = WM_JSON_TEMPLATES_VERSION;

    /** @var string */
    public $description;

    /** @var string */
    public $settings_exist = 'n';

    /** @var string */
    public $docs_url = '';

    /** @var array */
    public $settings;

    /** @var string[] */
    private $hooks = [
        'template_types',
    ];

    /**
     * wm_json_templates_ext constructor.
     *
     * @param string $settings
     */
    public function __construct($settings = '')
    {
        ee()->lang->loadfile('wm_json_templates');

        $this->name = lang('wm_json_templates');
        $this->description = lang('wm_json_templates_desc');
    }

    /**
     * @return array
     */
    public function template_types()
    {
        $custom_templates = ee()->extensions->last_call;

        if (!is_array($custom_templates)) {
            $custom_templates = [];
        }

        $custom_templates['json'] = [
            'template_name'           => 'JSON',
            'template_file_extension' => '.json',
            'template_headers'        => [
                'Content-Type: application/json',
            ],
        ];

        return $custom_templates;
    }

    /**
     * @return void
     */
    public function activate_extension()
    {
        foreach ($this->hooks as $hook) {
            ee()->db->insert('extensions', [
                'class'    => __CLASS__,
                'method'   => $hook,
                'hook'     => $hook,
                'settings' => serialize($this->settings),
                'priority' => 5,
                'version'  => $this->version,
                'enabled'  => 'y',
            ]);
        }
    }

    /**
     * @return false|null
     */
    public function update_extension($current = '')
    {
        if ($current === '' || $current === $this->version) {
            return false;
        }

        //if (version_compare($current, '1.0.0', '<')) {
        //    // Perform updates...
        //}

        ee()->db->where('class', __CLASS__);
        ee()->db->update('extensions', [
            'version' => $this->version,
        ]);
    }

    /**
     * @return void
     */
    public function disable_extension()
    {
        ee()->db->where('class', __CLASS__);
        ee()->db->delete('extensions');
    }
}
