<?php

namespace WarbleMedia\Seo\ControlPanel\Routes;

use ExpressionEngine\Library\CP\Table;
use ExpressionEngine\Service\Addon\Controllers\Mcp\AbstractRoute;
use function ee;

class AuditOverview extends AbstractRoute
{
    protected $route_path = 'audit_overview';
    protected $cp_page_title = 'Audit Overview';

    /**
     * @param false $id
     * @return AbstractRoute
     */
    public function process($id = false)
    {
        $this->addBreadcrumb('audit_overview', $this->cp_page_title);

        /** @var Table $table */
        $table = ee('CP/Table', [
            'autosort'   => true,
            'autosearch' => true,
        ]);

        $table->setColumns([
            'wm_seo_entry_id'   => ['type' => Table::COL_ID],
            'wm_seo_entry_title',
            'wm_seo_meta'       => ['encode' => false],
            'wm_seo_open_graph' => ['encode' => false],
            'wm_seo_twitter'    => ['encode' => false],
            'manage'            => ['type' => Table::COL_TOOLBAR],
        ]);

        $excludeChannels = ee()->config->item('wm_seo_exclude_channels') ?: [];

        $channelEntries = ee('Model')->get('ee:ChannelEntry')
            ->with('wm_seo:ContentFields as seo')
            ->filter('channel_id', 'NOT IN', $excludeChannels)
            ->fields(
                'entry_id',
                'title',
                'seo.title',
                'seo.description',
                'seo.robots_directive',
                'seo.og_title',
                'seo.og_type',
                'seo.og_url',
                'seo.og_description',
                'seo.og_image',
                'seo.twitter_title',
                'seo.twitter_type',
                'seo.twitter_description',
                'seo.twitter_image'
            )
            ->all();

        foreach ($channelEntries as $entry) {
            $entry->alias('wm_seo:ContentFields', 'ContentFields');
            $data[] = [
                $entry->getId(),
                $entry->title,
                $this->makeStatusCells([
                    ['label' => 'T', 'title' => 'Title', 'filled' => !empty($entry->ContentFields->title)],
                    ['label' => 'D', 'title' => 'Description', 'filled' => !empty($entry->ContentFields->description)],
                    ['label' => 'R', 'title' => 'Robots', 'filled' => !empty($entry->ContentFields->robots_directive)],
                ]),
                $this->makeStatusCells([
                    ['label' => 'T', 'title' => 'Title', 'filled' => !empty($entry->ContentFields->og_title)],
                    ['label' => 'D', 'title' => 'Description', 'filled' => !empty($entry->ContentFields->og_description)],
                    ['label' => 'T', 'title' => 'Type', 'filled' => !empty($entry->ContentFields->og_type)],
                    ['label' => 'U', 'title' => 'URL', 'filled' => !empty($entry->ContentFields->og_url)],
                    ['label' => 'I', 'title' => 'Image', 'filled' => !empty($entry->ContentFields->og_image)],
                ]),
                $this->makeStatusCells([
                    ['label' => 'T', 'title' => 'Title', 'filled' => !empty($entry->ContentFields->twitter_title)],
                    ['label' => 'D', 'title' => 'Description', 'filled' => !empty($entry->ContentFields->twitter_description)],
                    ['label' => 'T', 'title' => 'Type', 'filled' => !empty($entry->ContentFields->twitter_type)],
                    ['label' => 'I', 'title' => 'Image', 'filled' => !empty($entry->ContentFields->twitter_image)],
                ]),
                [
                    'toolbar_items' => [
                        'edit'     => [
                            'href'  => ee('CP/URL', 'publish/edit/entry/' . $entry->getId()),
                            'title' => lang('edit'),
                        ],
                        //'settings' => [
                        //    'href'  => ee('CP/URL')->make($this->getBaseUrl() . '/audit_entry/' . $entry->getId()),
                        //    'title' => lang('wm_seo_entry_audit'),
                        //],
                    ],
                ],
            ];
        }

        $table->setData($data);

        $vars['table'] = $table->viewData(ee('CP/URL')->make($this->getBaseUrl() . '/' . $this->route_path));

        $vars['pagination'] = ee('CP/Pagination', $vars['table']['total_rows'])
            ->perPage($vars['table']['limit'])
            ->currentPage($vars['table']['page'])
            ->render($vars['table']['base_url']);

        $this->setBody('audit_overview', $vars);

        return $this;
    }

    private function makeStatusCells(array $cells)
    {
        $status = '<div style="display: flex; column-gap: 5px">';

        foreach ($cells as $cell) {
            $color = $cell['filled'] ? 'var(--ee-success-light)' : 'var(--ee-error-light)';
            $status .= '<abbr style="background-color: ' . $color . '; width: 25px; padding: 5px; border-radius: 3px; text-decoration: none; text-align: center" title="' . $cell['title'] . '">';
            $status .= $cell['label'];
            $status .= '</abbr>';
        }

        $status .= '</div>';

        return $status;
    }
}
