<?php

namespace WarbleMedia\Seo\ControlPanel\Routes;

use ExpressionEngine\Library\CP\Form;
use ExpressionEngine\Service\Addon\Controllers\Mcp\AbstractRoute;
use WarbleMedia\Seo\Model\ConfigFields;
use function ee;
use function lang;

class Index extends AbstractRoute
{
    protected $route_path = 'index';
    protected $cp_page_title = 'Settings';
    protected $sidebar_priority = 1;

    /**
     * @param false $id
     * @return AbstractRoute
     */
    public function process($id = false)
    {
        $config = ee('Model')->get('wm_seo:ConfigFields')
            ->filter('site_id', ee()->config->item('site_id'))
            ->first();

        if ($config === null) {
            $config = ee('Model')->make('wm_seo:ConfigFields');
            $config->site_id = ee()->config->item('site_id');
        }

        if (ee('Request')->isPost()) {
            $config->set([
                'template'             => ee()->input->post('template'),
                'description'          => ee()->input->post('description'),
                'title_postfix'        => ee()->input->post('title_postfix'),
                'og_description'       => ee()->input->post('og_description'),
                'og_image'             => ee()->input->post('og_image'),
                'twitter_description'  => ee()->input->post('twitter_description'),
                'twitter_image'        => ee()->input->post('twitter_image'),
                'canonical_pagination' => ee()->input->post('canonical_pagination'),
            ]);

            // TODO: Validate $config

            $config->save();

            ee('CP/Alert')->makeInline('shared-form')
                ->asSuccess()
                ->withTitle(lang('success'))
                ->addToBody(lang('wm_seo_settings_updated'))
                ->defer();

            ee()->functions->redirect($this->url($this->route_path));
        }

        $this->addBreadcrumb('index', $this->cp_page_title);

        $vars['form'] = $this->getForm($config);

        $this->setBody('index', $vars);

        return $this;
    }

    private function getForm(ConfigFields $config)
    {
        /** @var Form $form */
        $form = ee('CP/Form');

        $defaults = $form->getGroup('Defaults');

        $defaults
            ->getFieldSet('title_postfix')
            ->setTitle('wm_seo_title_postfix')
            ->getField('title_postfix', 'text')
            ->setValue($config->title_postfix);

        $defaults
            ->getFieldSet('description')
            ->setTitle('wm_seo_default_description')
            ->getField('description', 'text')
            ->setValue($config->description);

        $defaults
            ->getFieldSet('canonical_pagination')
            ->setTitle('wm_seo_canonical_pagination')
            ->getField('canonical_pagination', 'yes_no')
            ->setValue($config->canonical_pagination);

        $openGraph = $form->getGroup('Open Graph');

        $openGraph
            ->getFieldSet('og_description')
            ->setTitle('wm_seo_default_og_description')
            ->setDesc('wm_seo_og_description_instructions')
            ->getField('og_description', 'text')
            ->setValue($config->og_description);

        $openGraph
            ->getFieldSet('og_image')
            ->setTitle('wm_seo_default_og_image')
            ->getField('og_image', 'file-picker')
            ->asImage()
            ->setValue($config->og_image);

        $twitter = $form->getGroup('Twitter');

        $twitter
            ->getFieldSet('twitter_description')
            ->setTitle('wm_seo_default_twitter_description')
            ->getField('twitter_description', 'text')
            ->setValue($config->twitter_description);

        $twitter
            ->getFieldSet('twitter_image')
            ->setTitle('wm_seo_default_twitter_image')
            ->getField('twitter_image', 'file-picker')
            ->asImage()
            ->setValue($config->twitter_image);

        $development = $form->getGroup('Development');

        $development
            ->getFieldSet('template')
            ->setTitle('wm_seo_template')
            ->getField('template', 'textarea')
            ->setRows(10)
            ->setValue($config->template);

        return $form->toArray();
    }
}
