<?php

namespace WarbleMedia\Seo\Tags;

use ExpressionEngine\Service\Addon\Controllers\Tag\AbstractRoute;
use function array_slice;
use function count;
use function ee;
use function htmlspecialchars;
use function implode;
use function preg_match;
use function preg_replace;
use function str_replace;
use function ucfirst;

class Output extends AbstractRoute
{
    // Example tag: {exp:wm_seo:output}
    public function process()
    {
        // Entry selection params
        $site_id = ee()->TMPL->fetch_param('site_id', ee()->config->item('site_id'));
        $entry_id = ee()->TMPL->fetch_param('entry_id');
        $channel = ee()->TMPL->fetch_param('channel');
        $url_title = ee()->TMPL->fetch_param('url_title');
        $category_url_title = ee()->TMPL->fetch_param('category_url_title');

        // Dynamic entry selection params
        $use_last_segment = ee()->TMPL->fetch_param('use_last_segment') === 'yes';
        $friendly_segments = ee()->TMPL->fetch_param('friendly_segments') === 'yes';
        $ignore_last_segments = ee()->TMPL->fetch_param('ignore_last_segments', false);

        $tag_prefix = ee()->TMPL->fetch_param('tag_prefix');

        // Default values params
        $default_title = ee()->TMPL->fetch_param('default_title');
        $default_description = ee()->TMPL->fetch_param('default_description');
        $default_og_description = ee()->TMPL->fetch_param('default_og_description');
        $default_og_image = ee()->TMPL->fetch_param('default_og_image');
        $default_twitter_description = ee()->TMPL->fetch_param('default_twitter_description');
        $default_twitter_image = ee()->TMPL->fetch_param('default_twitter_image');

        // Value manipulation params
        $title_prefix = ee()->TMPL->fetch_param('title_prefix');
        $title_postfix = ee()->TMPL->fetch_param('title_postfix');
        $title_separator = ee()->TMPL->fetch_param('title_separator');
        $title_override = ee()->TMPL->fetch_param('title_override');

        $canonical_override = ee()->TMPL->fetch_param('canonical');

        $config = ee('Model')->get('wm_seo:ConfigFields')
            ->filter('site_id', $site_id)
            ->first();

        if ($config === null) {
            $config = ee('Model')->make('wm_seo:ConfigFields');
        }

        $tagdata = ee()->TMPL->tagdata;

        if (empty($tagdata)) {
            $tagdata = $config->template;
        }

        if ($use_last_segment) {
            $url_title = $this->getUrlTitleFromSegment($ignore_last_segments);
        }

        $got_values = false;

        if ($category_url_title) {
            $category = ee('Model')->get('ChannelCategory')
                ->filter('site_id', $site_id)
                ->filter('cat_url_title', $category_url_title)
                ->first();

            if ($category !== null) {
                $tagdata = $this->clearExtraTags($tagdata, $tag_prefix);

                $vars = [
                    $tag_prefix . 'title'               => htmlspecialchars($this->getPreferedValue($category->cat_name, $default_title), ENT_QUOTES),
                    $tag_prefix . 'meta_description'    => htmlspecialchars($this->getPreferedValue($category->cat_description, $default_description, $config->description), ENT_QUOTES),
                    $tag_prefix . 'og_title'            => htmlspecialchars($this->getPreferedValue($category->cat_name, $default_title), ENT_QUOTES),
                    $tag_prefix . 'og_description'      => htmlspecialchars($this->getPreferedValue($default_og_description, $config->og_description), ENT_QUOTES),
                    $tag_prefix . 'og_image'            => $this->getPreferedValue($config->og_image, $default_og_image),
                    $tag_prefix . 'twitter_title'       => htmlspecialchars($this->getPreferedValue($category->cat_name, $default_title), ENT_QUOTES),
                    $tag_prefix . 'twitter_description' => htmlspecialchars($this->getPreferedValue($default_twitter_description, $config->twitter_description), ENT_QUOTES),
                    $tag_prefix . 'twitter_image'       => $this->getPreferedValue($config->twitter_image, $default_twitter_image),
                ];

                $got_values = true;
            }
        } elseif ($entry_id || $url_title) {
            //// if we're retrieving by url_title and not entry_id
            //if ($url_title && !$entry_id) {
            //    $pages = ee()->config->item('site_pages');
            //    if (isset($pages[$site_id]) && isset($pages[$site_id]['uris'])) {
            //        $current_uri_string = ee()->uri->uri_string();
            //        if ($current_uri_string !== '') {
            //            foreach ($pages[$site_id]['uris'] as $page_entry_id => $page_uri) {
            //                if (trim($page_uri, '/') === $current_uri_string) {
            //                    $entry_id = $page_entry_id;
            //                    $url_title = false; // pages will override - found entry_id so ignore url_title from now
            //                    $canonical_url = $this->getCanonicalUrl($site_id, $ignore_last_segments, $page_uri);
            //                }
            //            }
            //        }
            //    }
            //}

            $excludeChannels = ee()->config->item('wm_seo_exclude_channels') ?: [];

            $query = ee('Model')->get('ChannelEntry')
                ->with('wm_seo:ContentFields as seo')
                ->fields(
                    'entry_id',
                    'title',
                    'seo.title',
                    'seo.description',
                    'seo.robots_directive',
                    'seo.og_title',
                    'seo.og_type',
                    'seo.og_url',
                    'seo.og_description',
                    'seo.og_image',
                    'seo.twitter_title',
                    'seo.twitter_type',
                    'seo.twitter_description',
                    'seo.twitter_image'
                )
                ->filter('channel_id', 'NOT IN', $excludeChannels)
                ->filter('site_id', $site_id);

            if ($url_title) {
                $query->filter('url_title', $url_title);
            } else {
                $query->filter('entry_id', $entry_id);
            }

            if ($channel !== false) {
                $query
                    ->with('Channel')
                    ->filter('Channel.channel_name', $channel);
            }

            $entry = $query->first();

            if ($entry !== null) {
                $entry->alias('wm_seo:ContentFields', 'ContentFields');

                $vars = [
                    $tag_prefix . 'title' => htmlspecialchars($this->getPreferedValue($entry->ContentFields->title, $entry->title, $default_title), ENT_QUOTES),
                            $tag_prefix . 'meta_description'    => htmlspecialchars($this->getPreferedValue($entry->ContentField->description, $default_description, $config->description), ENT_QUOTES),
                            $tag_prefix . 'robots_directive'    => $this->getRobotsValue($entry->ContentField->robots_directive),
                            $tag_prefix . 'og_title'            => htmlspecialchars($this->getPreferedValue($entry->ContentField->og_title, $entry->title, $default_title), ENT_QUOTES),
                            $tag_prefix . 'og_description'      => htmlspecialchars($this->getPreferedValue($entry->ContentField->og_description, $default_og_description, $config->og_description), ENT_QUOTES),
                            $tag_prefix . 'og_type'             => $this->getOGType($entry->ContentField->og_type),
                            $tag_prefix . 'og_url'              => htmlspecialchars($entry->ContentField->og_url, ENT_QUOTES),
                            $tag_prefix . 'og_image'            => $this->getPreferedValue($entry->ContentField->og_image, $default_og_image, $config->og_image),
                            $tag_prefix . 'twitter_title'       => htmlspecialchars($this->getPreferedValue($entry->ContentField->twitter_title, $entry->title, $default_title), ENT_QUOTES),
                            $tag_prefix . 'twitter_description' => htmlspecialchars($this->getPreferedValue($entry->ContentField->twitter_description, $default_twitter_description, $config->twitter_description), ENT_QUOTES),
                            $tag_prefix . 'twitter_type'        => $this->getTwitterType($entry->ContentField->twitter_type),
                            $tag_prefix . 'twitter_image'       => $this->getPreferedValue($entry->ContentField->twitter_image, $default_twitter_image, $config->twitter_image),
                ];

                $got_values = true;
            }
        }

        if (!$got_values) {
            $tagdata = $this->clearExtraTags($tagdata, $tag_prefix);

            $vars = [
                $tag_prefix . 'title'               => htmlspecialchars($default_title, ENT_QUOTES),
                $tag_prefix . 'meta_description'    => htmlspecialchars($this->getPreferedValue($default_description, $config->description), ENT_QUOTES),
                $tag_prefix . 'og_title'            => htmlspecialchars($default_title, ENT_QUOTES),
                $tag_prefix . 'og_description'      => htmlspecialchars($this->getPreferedValue($default_og_description, $config->og_description), ENT_QUOTES),
                $tag_prefix . 'og_image'            => $this->getPreferedValue($config->og_image, $default_og_image),
                $tag_prefix . 'twitter_title'       => htmlspecialchars($default_title, ENT_QUOTES),
                $tag_prefix . 'twitter_description' => htmlspecialchars($this->getPreferedValue($default_twitter_description, $config->twitter_description), ENT_QUOTES),
                $tag_prefix . 'twitter_image'       => $this->getPreferedValue($config->twitter_image, $default_twitter_image),
            ];
        }

        if ($canonical_override) {
            $vars[$tag_prefix . 'canonical_url'] = $canonical_override;
        } else {
            $vars[$tag_prefix . 'canonical_url'] = $this->getCanonicalUrl($ignore_last_segments, $config->canonical_pagination !== 'n');
        }

        // Apply prefix and postfix to title
        $title = $vars[$tag_prefix . 'title'];
        if ($title !== '' && $title_postfix === false) {
            $title_postfix = str_replace('&nbsp;', ' ', $config->title_postfix);
        }

        if ($title_separator !== false) {
            $title_separator = ' ' . $title_separator . ' ';
        }

        $vars[$tag_prefix . 'entry_title'] = $title;
        $vars[$tag_prefix . 'title'] = html_entity_decode($title_prefix . $title . $title_postfix . $title_separator);

        // Segment variables are not parsed yet, so we do it ourselves if they are in use in the template
        if (preg_match_all('~\{segment_(\d)}~i', $tagdata, $matches)) {
            $tags = $matches[0];
            $count = count($tags);
            $segment_numbers = $matches[1];
            $word_separator_replace = ee()->config->item('word_separator') === 'underscore' ? '_' : '-';

            for ($i = 0; $i < $count; $i++) {
                $tag = $tags[$i];
                $segment_value = ee()->uri->segment($segment_numbers[$i]);

                if ($friendly_segments) {
                    $segment_value = ucfirst(str_replace($word_separator_replace, ' ', $segment_value));
                }

                $tagdata = str_replace($tag, $segment_value, $tagdata);
            }
        }

        // Apply title override
        if ($title_override) {
            $tagdata = preg_replace('~<title>([^<]*)</title>~', '<title>' . $title_override . '</title>', $tagdata);
        }

        ee()->load->library('typography');

        $tagdata = ee()->TMPL->parse_variables_row($tagdata, $vars);
        $tagdata = ee()->typography->parse_file_paths($tagdata);

        return $tagdata;
    }

    /**
     * Will clear all {extra:field_name} tags from the tagdata
     */
    private function clearExtraTags($tagdata, $tag_prefix)
    {
        return preg_replace('~\{' . $tag_prefix . 'extra:[^}]*}~', '', $tagdata);
    }

    /**
     * Get the last segment from the URL (ignore pagination in url)
     */
    private function getUrlTitleFromSegment($ignore_segments = false)
    {
        $segment_count = ee()->uri->total_segments();

        if ($this->isPaginationSegment(ee()->uri->segment($segment_count))) {
            $segment_count -= 1;
        }

        if ($ignore_segments) {
            $segment_count -= $ignore_segments;
        }

        if ($segment_count < 1) {
            $segment_count = 1;
        }

        return ee()->uri->segment($segment_count);
    }

    private function isPaginationSegment($segment)
    {
        return preg_match('~^P\d+$~', $segment);
    }

    private function getCanonicalUrl($ignore_segments, $include_pagination)
    {
        $pagination = '';
        $segment_count = ee()->uri->total_segments();

        if ($this->isPaginationSegment(ee()->uri->segment($segment_count))) {
            $pagination = ee()->uri->segment($segment_count);
            $segment_count -= 1;
        }

        if ($ignore_segments) {
            $segment_count -= $ignore_segments;
        }

        $segments = ee()->uri->segment_array();
        $segments = array_slice($segments, 0, $segment_count);

        if ($include_pagination) {
            $segments[] = $pagination;
        }

        return ee()->functions->create_url(implode('/', $segments));
    }

    private function getPreferedValue(...$values)
    {
        foreach ($values as $value) {
            if (!empty($value)) {
                return $value;
            }
        }

        return '';
    }

    private function getRobotsValue($value)
    {
        return match ($value) {
            0       => 'INDEX, FOLLOW',
            1       => 'NOINDEX, FOLLOW',
            2       => 'INDEX, NOFOLLOW',
            3       => 'NOINDEX, NOFOLLOW',
            default => '',
        };
    }

    private function getOGType($value)
    {
        return match ($value) {
            0       => 'article',
            1       => 'book',
            2       => 'music.song',
            3       => 'music.album',
            4       => 'music.playlist',
            5       => 'music.radio_station',
            6       => 'profile',
            7       => 'video.movie',
            8       => 'video.episode',
            9       => 'video.tv_show',
            10      => 'video.other',
            11      => 'website',
            default => '',
        };
    }

    private function getTwitterType($value)
    {
        return match ($value) {
            0       => 'summary',
            1       => 'summary_large_image',
            2       => 'player',
            default => '',
        };
    }
}
