<?php

use ExpressionEngine\Model\Channel\ChannelEntry;

class Wm_seo_tab
{
    public function __construct()
    {
        ee()->lang->loadfile('wm_seo');

        // Allow tab title override
        if (ee()->config->item('wm_seo_tab_title') !== false) {
            ee()->lang->language['wm_seo'] = ee()->config->item('wm_seo_tab_title');
        }
    }

    public function display($channel_id, $entry_id = '')
    {
        $settings = [];
        $contentFields = null;

        $excludeChannels = ee()->config->item('wm_seo_exclude_channels') ?: [];

        if (in_array($channel_id, $excludeChannels, false)) {
            return [
                'wm_seo_audit_disabled' => [
                    'field_id'             => 'wm_seo_audit_disabled',
                    'field_label'          => lang('wm_seo_audit_disabled'),
                    'field_required'       => 'n',
                    'field_show_fmt'       => 'n',
                    'field_instructions'   => lang('wm_seo_audit_disabled_details'),
                    'field_text_direction' => 'ltr',
                    'field_type'           => 'hidden',
                ],
            ];
        }

        if ($entry_id) {
            $contentFields = ee('Model')->get('wm_seo:ContentFields')
                ->filter('site_id', ee()->config->item('site_id'))
                ->filter('entry_id', $entry_id)
                ->first();
        }

        $settings['wm_seo_title'] = [
            'field_id'             => 'wm_seo_title',
            'field_label'          => lang('wm_seo_title'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_instructions'   => lang('wm_seo_title_instructions'),
            'field_data'           => $contentFields ? $contentFields->title : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 90,
        ];

        $settings['wm_seo_description'] = [
            'field_id'             => 'wm_seo_description',
            'field_label'          => lang('wm_seo_description'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_instructions'   => lang('wm_seo_description_instructions'),
            'field_data'           => $contentFields ? $contentFields->description : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 1024,
        ];

        $settings['wm_seo_robots_directive'] = [
            'field_id'             => 'wm_seo_robots_directive',
            'field_label'          => lang('wm_seo_robots_directive'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->robots_directive : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'select',
            'field_list_items'     => [
                'INDEX, FOLLOW',
                'NOINDEX, FOLLOW',
                'INDEX, NOFOLLOW',
                'NOINDEX, NOFOLLOW',
            ],
        ];

        $settings['wm_seo_og_title'] = [
            'field_id'             => 'wm_seo_og_title',
            'field_label'          => lang('wm_seo_og_title'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->og_title : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 90,
        ];

        $settings['wm_seo_og_description'] = [
            'field_id'             => 'wm_seo_og_description',
            'field_label'          => lang('wm_seo_og_description'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_instructions'   => lang('wm_seo_og_description_instructions'),
            'field_data'           => $contentFields ? $contentFields->og_description : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 1024,
        ];

        $settings['wm_seo_og_type'] = [
            'field_id'             => 'wm_seo_og_type',
            'field_label'          => lang('wm_seo_og_type'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->og_type : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'select',
            'field_list_items'     => [
                'Article',
                'Book',
                'Music - Song',
                'Music - Album',
                'Music - Playlist',
                'Music - Radio Station',
                'Profile',
                'Video - Movie',
                'Video - Episode',
                'Video - TV Show',
                'Video - Other',
                'Website',
            ],
        ];

        $settings['wm_seo_og_url'] = [
            'field_id'             => 'wm_seo_og_url',
            'field_label'          => lang('wm_seo_og_url'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->og_url : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 1024,
        ];

        $settings['wm_seo_og_image'] = [
            'field_id'             => 'wm_seo_og_image',
            'field_label'          => lang('wm_seo_og_image'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->og_image : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'file',
        ];

        $settings['wm_seo_twitter_title'] = [
            'field_id'             => 'wm_seo_twitter_title',
            'field_label'          => lang('wm_seo_twitter_title'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->twitter_title : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 90,
        ];

        $settings['wm_seo_twitter_description'] = [
            'field_id'             => 'wm_seo_twitter_description',
            'field_label'          => lang('wm_seo_twitter_description'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->twitter_description : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'text',
            'field_maxl'           => 1024,
        ];

        $settings['wm_seo_twitter_type'] = [
            'field_id'             => 'wm_seo_twitter_type',
            'field_label'          => lang('wm_seo_twitter_type'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->twitter_type : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'select',
            'field_list_items'     => [
                'Summary',
                'Summary - Large Image',
                'Player',
            ],
        ];

        $settings['wm_seo_twitter_image'] = [
            'field_id'             => 'wm_seo_twitter_image',
            'field_label'          => lang('wm_seo_twitter_image'),
            'field_required'       => 'n',
            'field_show_fmt'       => 'n',
            'field_data'           => $contentFields ? $contentFields->twitter_image : '',
            'field_text_direction' => 'ltr',
            'field_type'           => 'file',
        ];

        //if ($contentFields !== null) {
        //    $auditUrl = ee('CP/URL', 'addons/settings/wm_seo/audit_entry')
        //        ->setQueryStringVariable('entry_id', $contentFields->entry_id);
        //
        //    $settings['wm_seo_audit_btn'] = [
        //        'field_id'             => 'wm_seo_audit_btn',
        //        'field_label'          => lang('wm_seo_audit_btn'),
        //        'field_required'       => 'n',
        //        'field_show_fmt'       => 'n',
        //        'field_instructions'   => '<a class="button button--primary" href="' . $auditUrl . '" target="_blank"><i class="fas fa-clipboard-check"></i> ' . lang('wm_seo_audit_btn_label') . '</a>',
        //        'field_text_direction' => 'ltr',
        //        'field_type'           => 'hidden',
        //    ];
        //}

        return $settings;
    }

    public function validate($entry, $values)
    {
        $validator = ee('Validation')->make([
            //'wm_seo_title'               => '',
            //'wm_seo_description'         => '',
            //'wm_seo_robots_directive'    => '',
            //'wm_seo_og_title'            => '',
            //'wm_seo_og_description'      => '',
            //'wm_seo_og_type'             => '',
            //'wm_seo_og_url'              => '',
            //'wm_seo_og_image'            => '',
            //'wm_seo_twitter_title'       => '',
            //'wm_seo_twitter_description' => '',
            //'wm_seo_twitter_type'        => '',
            //'wm_seo_twitter_image'       => '',
        ]);

        return $validator->validate($values);
    }

    public function cloneData(ChannelEntry $entry, $values)
    {
        return $values;
    }

    public function save(ChannelEntry $entry, $values)
    {
        $contentFields = ee('Model')->get('wm_seo:ContentFields')
            ->filter('site_id', $entry->site_id)
            ->filter('entry_id', $entry->entry_id)
            ->first();

        if ($contentFields === null) {
            $contentFields = ee('Model')->make('wm_seo:ContentFields');
        }

        $contentFields->set([
            'site_id'             => $entry->site_id,
            'entry_id'            => $entry->entry_id,
            'title'               => $values['wm_seo_title'],
            'description'         => $values['wm_seo_description'],
            'robots_directive'    => $values['wm_seo_robots_directive'],
            'og_title'            => $values['wm_seo_og_title'],
            'og_description'      => $values['wm_seo_og_description'],
            'og_type'             => $values['wm_seo_og_type'],
            'og_url'              => $values['wm_seo_og_url'],
            'og_image'            => $values['wm_seo_og_image'],
            'twitter_title'       => $values['wm_seo_twitter_title'],
            'twitter_description' => $values['wm_seo_twitter_description'],
            'twitter_type'        => $values['wm_seo_twitter_type'],
            'twitter_image'       => $values['wm_seo_twitter_image'],
        ]);

        $contentFields->save();
    }

    public function delete($entry_ids)
    {
        ee('Model')->get('wm_seo:ContentFields')
            ->filter('site_id', ee()->config->item('site_id'))
            ->filter('entry_id', 'IN', $entry_ids)
            ->delete();
    }
}
