<?php

namespace Zipscreen\Extensions;

use ExpressionEngine\Model\Channel\ChannelEntry;
use ExpressionEngine\Service\Addon\Controllers\Extension\AbstractRoute;
use Solspace\Addons\FreeformNext\Library\Composer\Components\Form;
use Solspace\Addons\FreeformNext\Model\SubmissionModel;
use function array_filter;
use function ee;
use function file_get_contents;
use function file_put_contents;
use function implode;
use function json_decode;
use function json_encode;
use function reset;
use function sprintf;
use function trim;
use function urlencode;

class FreeformNextFormAfterSubmit extends AbstractRoute
{
    public function process(Form $form, SubmissionModel $submission)
    {
        if ($form->getHandle() !== 'join_network') {
            return;
        }

        $title = $submission->getFieldValue('company');
        $slug = ee('Format')->make('Text', $title)->urlSlug()->compile();
        $timestamp = ee()->localize->now;

        $address = [
            'street'      => $submission->getFieldValue('street_1'),
            'street_2'    => $submission->getFieldValue('street_2'),
            'city'        => $submission->getFieldValue('suburb'),
            'region'      => $submission->getFieldValue('state'),
            'postal_code' => $submission->getFieldValue('postcode'),
            'country'     => 'au',
        ];

        $query = $this->getUrlEncodedAddress($address);
        $geocode = $this->getGeocodingResponse($query);

        //if ($geocode === false) {
        //    return;
        //}

        /** @var ChannelEntry $entry */
        $entry = ee('Model')->make('ChannelEntry');
        $entry->author_id = 1;
        $entry->channel_id = 2;
        $entry->title = $title;
        $entry->url_title = sprintf('%s-%d', $slug, $timestamp);
        $entry->status = 'closed';
        $entry->entry_date = $timestamp;

        $entry->field_id_10 = $submission->getFieldValue('company');
        $entry->field_id_3 = $submission->getFieldValue('phone');
        $entry->field_id_4 = $submission->getFieldValueAsString('email');
        $entry->field_id_5 = ee('Format')->make('Text', $submission->getFieldValue('website'))->url();
        $entry->field_id_2 = $address;
        $entry->field_id_8 = is_array($geocode) ? $geocode['geometry']['location']['lat'] : 'COULD NOT PARSE ADDRESS';
        $entry->field_id_9 = is_array($geocode) ? $geocode['geometry']['location']['lng'] : 'COULD NOT PARSE ADDRESS';
        $entry->field_id_6 = $submission->getFieldValue('type');
        $entry->field_id_7 = $submission->getFieldValue('service');

        $result = $entry->validate();

        if ($result->isValid()) {
            $entry->save();
        } else {
            file_put_contents(__DIR__ . '/validation.log', 'Failed validation. (entry: ' . $entry->toJson() . ') (errors: ' . json_encode($result->getAllErrors()) . ')', FILE_APPEND);
        }
    }

    private function getUrlEncodedAddress(array $data): string
    {
        $output = [];

        if ($data['street']) {
            $output[] = $data['street'];
        }

        if ($data['street_2']) {
            $output[] = $data['street_2'];
        }

        $output[] = trim("{$data['city']}, {$data['region']} {$data['postal_code']}", ' ,');

        return urlencode(implode(', ', array_filter($output)));
    }

    private function getGeocodingResponse(string $query): array|false
    {
        $apiKey = ee()->config->item('wm_address_gmaps_api_key');

        $request = sprintf('https://maps.googleapis.com/maps/api/geocode/json?address=%s&key=%s', $query, $apiKey);

        $response = file_get_contents($request);
        $response = json_decode($response, true, 512, JSON_THROW_ON_ERROR);

        if (!isset($response['status']) || $response['status'] !== 'OK' || empty($response['results'])) {
            file_put_contents(__DIR__ . '/geocoding.log', 'Could not parse address "' . json_encode($query) . '". ' . json_encode($response), FILE_APPEND);
            return false;
        }

        return reset($response['results']);
    }
}
